
from epyk.core import html
from epyk.core.js.packages import JsFontAwesome


class Icons(object):
  def __init__(self, context):
    self.context = context

  @property
  def get(self):
    return JsFontAwesome

  def awesome(self, icon, text=None, tooltip=None, position=None, width=(25, 'px'), height=(25, 'px'),
              htmlCode=None, profile=None):
    """
    Description:
    ------------

    Usage::

      rptObj.ui.icons.awesome(icon="fas fa-align-center")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param position: Optional. The position of the icon in the line (left, right, center)
    :param tooltip: Optional. A string with the value of the tooltip
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param profile: Optional. A flag to set the component performance storage
    """
    html_edit = html.HtmlButton.IconEdit(self.context.rptObj, position, icon, text, tooltip, width, height, htmlCode, profile)
    html_edit.style.css.float = position
    html_edit.style.css.display = "inline-block"
    return html_edit

  def edit(self, text=None, position=None, tooltip="Edit", width=(None, 'px'), height=(None, 'px'), htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `far fa-edit <https://fontawesome.com/icons/edit>`_ icon

    Usage::

      rptObj.ui.icons.edit()
      rptObj.ui.icons.edit().color("red")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param position: Optional. The position of the icon in the line (left, right, center)
    :param tooltip: Optional. A string with the value of the tooltip
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param profile: Optional. A flag to set the component performance storage
    """
    return self.awesome('far fa-edit', text, tooltip, position, width, height, htmlCode, profile)

  def clock(self, text=None, position=None, tooltip="Last Updated Time", width=(None, 'px'), height=(None, 'px'),
            htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `fas fa-clock <https://fontawesome.com/icons/clock>`_ icon

    Usage::

      rptObj.ui.icons.clock()
      rptObj.ui.icons.clock().color("red")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param position: Optional. The position of the icon in the line (left, right, center)
    :param tooltip: Optional. A string with the value of the tooltip
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param profile: Optional. A flag to set the component performance storage
    """
    return self.awesome('fas fa-clock', text, tooltip, position, width, height, htmlCode, profile)

  def refresh(self, text=None, position=None, tooltip="Refresh Component", width=(None, 'px'), height=(None, 'px'),
              htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `fas fa-sync-alt <https://fontawesome.com/icons/sync-alt>`_ icon

    Usage::

      rptObj.ui.icons.refresh()
      rptObj.ui.icons.refresh().color("red")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param position: Optional. The position of the icon in the line (left, right, center)
    :param tooltip: Optional. A string with the value of the tooltip
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param profile: Optional. A flag to set the component performance storage
    """
    return self.awesome('fas fa-sync-alt', text, tooltip, position, width, height, htmlCode, profile)

  def pdf(self, text=None, position=None, tooltip="Convert to PDF", width=(None, 'px'), height=(None, 'px'),
          htmlCode=None,  profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `far fa-file-pdf <https://fontawesome.com/icons/file-pdf>`_ icon

    Usage::

      rptObj.ui.icons.pdf(tooltip="helper")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param position: Optional. The position of the icon in the line (left, right, center)
    :param tooltip: Optional. A string with the value of the tooltip
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param profile: Optional. A flag to set the component performance storage
    """
    return self.awesome('far fa-file-pdf', text, tooltip, position, width, height, htmlCode, profile)

  def plus(self, text=None, position=None, tooltip="Add line", width=(None, 'px'), height=(None, 'px'),
           htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `far fa-file-pdf <https://fontawesome.com/icons/plus-square>`_ icon

    Usage::

      rptObj.ui.icons.plus()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param position: Optional. The position of the icon in the line (left, right, center)
    :param tooltip: Optional. A string with the value of the tooltip
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param profile: Optional. A flag to set the component performance storage
    """
    return self.awesome('fas fa-plus-square', text, tooltip, position, width, height, htmlCode, profile)

  def excel(self, text=None, position=None, tooltip="Convert to Excel", width=(None, 'px'), height=(None, 'px'),
            htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `far fa-file-excel <https://fontawesome.com/icons/file-excel>`_ icon

    Usage::

      rptObj.ui.icons.excel()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param position: Optional. The position of the icon in the line (left, right, center)
    :param tooltip: Optional. A string with the value of the tooltip
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param profile: Optional. A flag to set the component performance storage
    """
    return self.awesome('far fa-file-excel', text, tooltip, position, width, height, htmlCode, profile)

  def download(self, text=None, position=None, tooltip="Download", width=(None, 'px'), height=(None, 'px'),
               htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `fas fa-download <https://fontawesome.com/icons/download>`_ icon

    Usage::

      rptObj.ui.icons.download()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param position: Optional. The position of the icon in the line (left, right, center)
    :param tooltip: Optional. A string with the value of the tooltip
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param profile: Optional. A flag to set the component performance storage
    """
    return self.awesome('fas fa-download', text, tooltip, position, width, height, htmlCode, profile)

  def delete(self, text=None, position=None, tooltip="Delete Component on the page", width=(None, 'px'),
             height=(None, 'px'), htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `far fa-trash-alt <https://fontawesome.com/icons/trash-alt>`_ icon

    Usage::

      rptObj.ui.icons.delete()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param position: Optional. The position of the icon in the line (left, right, center)
    :param tooltip: Optional. A string with the value of the tooltip
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param profile: Optional. A flag to set the component performance storage
    """
    return self.awesome('far fa-trash-alt', text, tooltip, position, width, height, htmlCode, profile)

  def zoom(self, text=None, position=None, tooltip="Zoom on Component", width=(None, 'px'), height=(None, 'px'), htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `fas fa-search-plus <https://fontawesome.com/icons/search-plus>`_ icon

    Usage::

      rptObj.ui.icons.zoom()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param position: Optional. The position of the icon in the line (left, right, center)
    :param tooltip: Optional. A string with the value of the tooltip
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param profile: Optional. A flag to set the component performance storage
    """
    return self.awesome('fas fa-search-plus', text, tooltip, position, width, height, htmlCode, profile)

  def capture(self, text=None, position=None, tooltip="Save to clipboard", width=(None, 'px'), height=(None, 'px'),
              htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `far fa-clipboard <https://fontawesome.com/icons/clipboard>`_ icon

    Usage::

      rptObj.ui.icons.capture()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param position: Optional. The position of the icon in the line (left, right, center)
    :param tooltip: Optional. A string with the value of the tooltip
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param profile: Optional. A flag to set the component performance storage
    """
    return self.awesome('far fa-clipboard', text, tooltip, position, width, height, htmlCode, profile)

  def remove(self, text=None, position=None, tooltip="Remove Item", width=(None, 'px'), height=(None, 'px'),
             htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `fas fa-times-circle <https://fontawesome.com/icons/times-circle>`_ icon

    Usage::

      rptObj.ui.icons.remove()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param position: Optional. The position of the icon in the line (left, right, center)
    :param tooltip: Optional. A string with the value of the tooltip
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param profile: Optional. A flag to set the component performance storage
    """
    return self.awesome('fas fa-times-circle', text, tooltip, position, width, height, htmlCode, profile)

  def table(self, text=None, position=None, tooltip="Convert to Table", width=(None, 'px'), height=(None, 'px'),
            htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `fas fa-table <https://fontawesome.com/icons/table>`_ icon

    Usage::

      rptObj.ui.icons.table(tooltip="helper")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param position: Optional. The position of the icon in the line (left, right, center)
    :param tooltip: Optional. A string with the value of the tooltip
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param profile: Optional. A flag to set the component performance storage
    """
    return self.awesome('fas fa-table', text, tooltip, position, width, height, htmlCode, profile)

  def wrench(self, text=None, position=None, tooltip="Processing Time", width=(None, 'px'), height=(None, 'px'),
             htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `fas fa-wrench <https://fontawesome.com/icons/wrench>`_ icon

    Usage::

      rptObj.ui.icons.wrench()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param position: Optional. The position of the icon in the line (left, right, center)
    :param tooltip: Optional. A string with the value of the tooltip
    :param width: Optional. A tuple with the integer for the component width and its unit
    :param height: Optional. A tuple with the integer for the component height and its unit
    :param htmlCode: Optional. An identifier for this component (on both Python and Javascript side)
    :param profile: Optional. A flag to set the component performance storage
    """
    return self.awesome('fas fa-wrench', text, tooltip, position, width, height, htmlCode, profile)

  def facebook(self, text=None, url="https://en-gb.facebook.com/", position=None, tooltip="Facebook", width=(25, 'px'),
               htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `fab fa-facebook-f <https://fontawesome.com/icons/facebook-f>`_ icon

    Usage::

      rptObj.ui.icons.facebook()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param text:
    :param url:
    :param position:
    :param tooltip:
    :param width:
    :param htmlCode:
    :param profile:
    """
    icon = self.awesome('fab fa-facebook-f', text, tooltip, position, width, width, htmlCode, profile)
    icon.css({"border-radius": "%spx" % width[0], "text-align": "center", "line-height": '%s%s' % (width[0], width[1])})
    icon.icon.css({"margin-right": "auto", "margin": "auto", "color": 'blue'})
    icon.style.add_classes.div.background_hover()
    icon.click([self.context.rptObj.js.navigateTo(url)])
    return icon

  def twitter(self, text=None, url="https://twitter.com", position=None, tooltip="Twitter", width=(25, 'px'),
              htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `fab fa-twitter <https://fontawesome.com/icons/twitter>`_ icon

    Usage::

      rptObj.ui.icons.twitter()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param text:
    :param url:
    :param position:
    :param tooltip:
    :param width:
    :param htmlCode:
    :param profile:
    """
    icon = self.awesome('fab fa-twitter', text, tooltip, position, width, width, htmlCode, profile)
    icon.css({"border-radius": "%spx" % width[0], "text-align": "center", "line-height": '%s%s' % (width[0], width[1])})
    icon.icon.css({"margin-right": "auto", "margin": "auto", "color": 'blue'})
    icon.style.add_classes.div.background_hover()
    icon.click([self.context.rptObj.js.navigateTo(url)])
    return icon

  def linkedIn(self, text=None, url="https://www.linkedin.com/home/?originalSubdomain=uk", position=None, tooltip="linkedIn",
               width=(25, 'px'), htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `fab fa-linkedin-in <https://fontawesome.com/icons/linkedin-in>`_ icon

    Usage::

      rptObj.ui.icons.linkedIn()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param text:
    :param url:
    :param position:
    :param tooltip:
    :param width:
    :param htmlCode:
    :param profile:
    """
    icon = self.awesome('fab fa-linkedin-in', text, tooltip, position, width, width, htmlCode, profile)
    icon.css({"border-radius": "%spx" % width[0], "text-align": "center", "line-height": '%s%s' % (width[0], width[1])})
    icon.icon.css({"margin-right": "auto", "margin": "auto", "color": 'blue'})
    icon.style.add_classes.div.background_hover()
    icon.click([self.context.rptObj.js.navigateTo(url)])
    return icon

  def youtube(self, text=None, url="https://www.youtube.com/", position=None, tooltip="Follow us on Youtube",
              width=(25, 'px'), htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `fab fa-youtube <https://fontawesome.com/icons/youtube>`_ icon

    Usage::

      rptObj.ui.icons.youtube()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param text:
    :param url:
    :param position:
    :param tooltip:
    :param width:
    :param htmlCode:
    :param profile:
    """
    icon = self.awesome('fab fa-youtube', text, tooltip, position, width, width, htmlCode, profile)
    icon.css({"border-radius": "%spx" % width[0], "text-align": "center", "line-height": '%s%s' % (width[0], width[1])})
    icon.icon.css({"margin-right": "auto", "margin": "auto", "color": 'blue'})
    icon.style.add_classes.div.background_hover()# addCls("CssDivOnHoverBackgroundLight")
    icon.click([self.context.rptObj.js.navigateTo(url)])
    return icon

  def github(self, text=None, url="https://github.com/", position=False, tooltip="Go the the Github project",
             width=(25, 'px'), htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `fab fa-github <https://fontawesome.com/icons/github>`_ icon

    Usage::

      rptObj.ui.icons.github()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param text:
    :param url:
    :param position:
    :param tooltip:
    :param width:
    :param htmlCode:
    :param profile:
    """
    icon = self.awesome('fab fa-github', text, tooltip, position, width, width, htmlCode, profile)
    icon.css({"border-radius": "%spx" % width[0], "text-align": "center", "line-height": '%s%s' % (width[0], width[1])})
    icon.icon.css({"margin-right": "auto", "margin": "auto", "color": 'blue'})
    icon.style.add_classes.div.background_hover()
    icon.click([self.context.rptObj.js.navigateTo(url)])
    return icon

  def python(self, text=None, url="https://pypi.org/", position=None, tooltip="Like or package",
             width=(25, 'px'), htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `fab fa-python <https://fontawesome.com/icons/python>`_ icon

    Usage::

      rptObj.ui.icons.python()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param text:
    :param url:
    :param position:
    :param tooltip:
    :param width:
    :param htmlCode:
    :param profile:
    """
    icon = self.awesome("fab fa-python", text, tooltip, position, width, width, htmlCode, profile)
    icon.css({"border-radius": "%spx" % width[0], "text-align": "center", "line-height": '%s%s' % (width[0], width[1])})
    icon.icon.css({"margin-right": "auto", "margin": "auto", "color": 'blue'})
    icon.style.add_classes.div.background_hover()
    icon.click([self.context.rptObj.js.navigateTo(url)])
    return icon

  def stackoverflow(self, text=None, url="https://stackoverflow.com/", position=None, tooltip="Share your comments",
                    width=(25, 'px'), htmlCode=None, profile=None):
    """
    Description:
    ------------

    Same as :func:`epyk.interfaces.components.CompIcons.Icons.awesome` with a `fab fa-stack-overflow <https://fontawesome.com/icons/stack-overflow>`_ icon

    Usage::

      rptObj.ui.icons.stackoverflow()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlButton.IconEdit`

    Attributes:
    ----------
    :param text:
    :param url:
    :param position:
    :param tooltip:
    :param width:
    :param htmlCode:
    :param profile:
    """
    icon = self.awesome("fab fa-stack-overflow", text, tooltip, position, width, width, htmlCode, profile)
    icon.css({"border-radius": "%spx" % width[0], "text-align": "center", "line-height": '%s%s' % (width[0], width[1])})
    icon.icon.css({"margin-right": "auto", "margin": "auto", "color": 'blue'})
    icon.style.add_classes.div.background_hover()
    icon.click([self.context.rptObj.js.navigateTo(url)])
    return icon

  def tick(self, flag=True, text=None, icons=(JsFontAwesome.ICON_CHECK, JsFontAwesome.ICON_TIMES), position=None,
           tooltip="", width=(None, 'px'), htmlCode=None, options=None, profile=None):
    """
    Description:
    ------------

    Usage::

      rptObj.ui.icons.tick()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlRadio.Tick`

    Attributes:
    ----------
    :param text:
    :param icons:
    :param position:
    :param tooltip:
    :param width:
    :param htmlCode:
    :param profile:
    """
    dftl_options = {"true": icons[0], "false": icons[1]}
    dftl_options.update(options or {})
    # report, position, icon, text, tooltip, width, height, htmlCode, profile
    icon = html.HtmlRadio.Tick(self.context.rptObj, position, icons[0] if flag else icons[1], text, tooltip, width, width,
                               htmlCode, dftl_options, profile)
    icon.click([
      icon.icon.dom.switchClass(icons[0] if flag else icons[1], icons[1] if flag else icons[0]),
      icon.icon.dom.transition('background', self.context.rptObj.theme.success[0], duration=.2, reverse=True)
    ])
    return icon

  def epyk(self, align="center", format='logo'):
    """
    Description:
    ------------
    Add the Epyk Icon

    Usage::

      rptObj.ui.icons.epyk()

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlImage.Image`

    Attributes:
    ----------
    :param align:
    :param format:
    :rtype: html.HtmlImage.Image
    """
    if format == 'logo':
      img, width, height = "epyklogo.ico", (32, 'px'), (32, 'px')
    elif format == 'small':
      img, width, height = "epyklogo_whole_small.png", (45, 'px'), (32, 'px')
    else:
      img, width, height = "epyklogo_whole_big.png", ("auto", ''), ('auto', '')
    icon = self.context.rptObj.ui.img(img, path="https://raw.githubusercontent.com/epykure/epyk-ui/master/epyk/static/images",
                                      align=align, width=width, height=height)
    icon.css({"text-align": "center", "padding": "auto", "vertical-align": "middle"})
    icon.style.add_classes.div.background_hover()
    return icon

  def signin(self, text, width=(40, "px"), icon=None):
    """
    Description:
    ------------

    Usage::

      rptObj.ui.icons.signin("test")

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlEvent.SignIn

    Attributes:
    ----------

    """
    bar = html.HtmlEvent.SignIn(self.context.rptObj, text, width, icon)
    return bar

  def bar(self, records=None, color=None, width=(70, 'px'), height=(None, 'px'), options=None):
    """
    Description:
    ------------
    Add a bespoke options / actions bar with icons

    Usage::

      Related Pages:
Attributes:
    ----------
    :param records:
    :param color:
    :param width:
    :param height:
    :param options:
    """
    records = records or []
    options = options or {}
    html_opts = html.HtmlEvent.OptionsBar(self.context.rptObj, records, width, height, color, options)
    return html_opts

  def avatar(self, img, name=None, width=(30, 'px'), height=(None, ''), options=None, profile=None):
    """
    Description:
    ------------

Attributes:
    ----------
    :param img:
    :param width:
    :param height:
    :param options:
    :param profile:
    """
    img = img.replace("\\", "/")
    if height[0] is None:
      height = width
    avatar = self.context.rptObj.ui.div("&nbsp;", width=width, height=height, options=options, profile=profile)
    avatar.css({"padding": '5px', 'border-radius': '30px',
              'background-repeat': 'no-repeat',
              'background-position': 'center', 'background-size': 'cover', 'cursor': 'pointer',
              'background-image': 'url(%s)' % img})
    if name is not None:
      avatar.tooltip(name)
    return avatar

  def badge(self, text="", icon=None, width=(25, "px"), height=(25, "px"), background_color=None, color=None, url=None,
            tooltip=None, options=None, profile=None):
    """
    Description:
    ------------
    Display a badge component using Bootstrap

    Usage::

      rptObj.ui.images.badge("Test badge", "Label", icon="fas fa-align-center")
      rptObj.ui.images.badge("This is a badge", background_color="red", color="white")
      rptObj.ui.images.badge(12, icon="far fa-bell", options={"badge_position": 'right'})

      b = rptObj.ui.images.badge(7688, icon="fab fa-python", options={'badge_css': {'color': 'white', "background": 'red'}})
      b.options.badge_css = {"background": 'green'}

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlImage.Badge`

    Related Pages:

        https://getbootstrap.com/docs/4.0/components/badge/

    Attributes:
    ----------
    :param text: The content of the badge
    :param icon: Optional, A String with the icon to display from font-awesome
    :param background_color: Optional, The background color of the badge
    :param color: Optional, The text color of the badge
    :param url:
    :param tooltip: Optional, The text to display in the tooltip
    :param options:
    :param profile: Optional, A boolean to store the performances for each components
    """
    if background_color is None:
      background_color = self.context.rptObj.theme.greys[0]
    if color is None:
      color = self.context.rptObj.theme.success[1]
    html_badge = html.HtmlImage.Badge(self.context.rptObj, text, width, height, None, icon, background_color, color, url,
                                      tooltip, options or {}, profile)
    return html_badge



from epyk.core.html import Html
from epyk.core.html.options import OptJsonFormatter

from epyk.core.js.html import JsHtmlStars
from epyk.core.js.packages import JsJsonFormatter
from epyk.core.js.primitives import JsObjects

from epyk.core.js import JsUtils

# The list of CSS classes
from epyk.core.css.styles import GrpClsLayout


class Hr(Html.Html):
  name = 'Line delimiter'

  def __init__(self, report, background_color, height, align, profile):
    super(Hr, self).__init__(report, "", profile=profile, css_attrs={"height": height,
                             'border-color': background_color or report.theme.greys[2],
                             'background-color': background_color or report.theme.greys[2]})
    if align == "center":
      self.style.css.margin = "auto"

  @property
  def style(self):
    """
    Description:
    ------------
    Property to the CSS Style of the component

    :return: GrpClsLayout.ClassStandard
    """
    if self._styleObj is None:
      self._styleObj = GrpClsLayout.ClassStandard(self)
    return self._styleObj

  def __str__(self):
    return '<hr %s>' % (self.get_attrs(pyClassNames=self.style.get_classes()))


class Newline(Html.Html):
  name = 'New line'

  def __str__(self):
    return "".join(['<br />'] * self.val)


class Stars(Html.Html):
  name = 'Stars'
  requirements = ('font-awesome', )

  def __init__(self, report, val, label, color, align, best, htmlCode, helper, profile):
    super(Stars, self).__init__(report, val, htmlCode=htmlCode, profile=profile)
    # Add the HTML components
    self._spans = []
    self._jsStyles = {'color': self._report.theme.success[1] if color is None else color}
    for i in range(best):
      self.add_span("", position="after", css=False)
      self._sub_htmls[-1].attr['class'].add("fa fa-star")
      self._sub_htmls[-1].css({"margin": 0, "padding": 0})
      self._sub_htmls[-1].set_attrs(name="data-level", value=i)
      if i < val:
        self._sub_htmls[-1].css({"color": self._jsStyles['color']})
      self._spans.append(self._sub_htmls[-1])
    self.set_attrs(name='data-level', value=val)
    self.add_label(label, {"margin": "0 0 0 5px", 'height': 'none', "text-align": "left", "display": "inline-block",
                           'float': 'None'}, position="after")
    self.add_helper(helper)
    if self.helper:
      self.helper.css({"margin": '1px 4px'})
    self.css({'text-align': align, "display": 'block'})

  @property
  def dom(self):
    """
    Description:
    ------------
    The JavaScript dom object to be used in any events

    :rtype: JsHtmlStars.Stars
    """
    if self._dom is None:
      self._dom = JsHtmlStars.Stars(self, report=self._report)
    return self._dom

  def click(self, js_fncs=None, profile=False, source_event=None):
    """
    Description:
    ------------
    Add the event click and double click to the starts item
    The Javascript function will be triggered after the change of content of the component

    Usage::

      stars = rptObj.ui.rich.stars(3, label="test Again")
    stars.click(rptObj.js.console.log("test").toStr())

    Attributes:
    ----------
    :param js_fncs: An array of Js functions or string. Or a string with the Js
    :param profile: Boolean.

    :return: self to allow the chains
    """
    self.css({"cursor": "pointer"})
    if js_fncs is None:
      js_fncs = []
    else:
      if not isinstance(js_fncs, list):
        js_fncs = [js_fncs]
    js_fncs = ["var data = parseInt(event.target.dataset.level)+1",
               self.build(data=JsObjects.JsObjects.get("data"), options=self._jsStyles)] + js_fncs
    str_fncs = JsUtils.jsConvertFncs(js_fncs)
    for span in self._spans:
      span.click(str_fncs, profile, source_event=source_event)
    return self

  @property
  def _js__builder__(self):
    return '''
      htmlObj.dataset.level = data;
      htmlObj.querySelectorAll("span").forEach(function(span, i){
        if (i < data){span.style.color = options.color}
        else {span.style.color = ''}})'''

  def __str__(self):
    return "<div %s>%s</div>" % (self.get_attrs(pyClassNames=self.style.get_classes()), self.helper)


class Help(Html.Html):
  requirements = ('font-awesome', )
  name = 'Info'

  def __init__(self, report, val, width, profile, options):
    super(Help, self).__init__(report, val, css_attrs={"width": width}, profile=profile)
    self.attr['class'].add("fas fa-question-circle")
    self.attr['title'] = val
    self._jsStyles = options

  @property
  def style(self):
    """
    Description:
    ------------
    Property to the CSS Style of the component

    :rtype: GrpClsLayout.ClassHelp
    """
    if self._styleObj is None:
      self._styleObj = GrpClsLayout.ClassHelp(self)
    return self._styleObj

  @property
  def _js__builder__(self):
    return '''
      htmlObj.setAttribute("title", data);
      if(typeof options.css !== 'undefined'){for(var k in options.css){htmlObj.style[k] = options.css[k]}}'''

  def __str__(self):
    return '<i %s></i>' % self.get_attrs(pyClassNames=self.style.get_classes())


class Loading(Html.Html):
  requirements = ('font-awesome', )
  name = 'Loading'

  def __init__(self, report, text, color, size, options):
    super(Loading, self).__init__(report, text)
    self.color = self._report.theme.greys[-1] if color is None else color
    self.size = size[0]
    self.css({'color': self.color, 'font-size': "%s%s" % (size[0], size[1]), 'z-index': 5, 'margin': 0})
    self.add_icon("fas fa-spinner fa-spin", css={"font-size": "%spx" % (self.size+8)})
    if options.get('fixed', False):
      self.icon.css({"margin-right": '5px', "font-size": 'inherit'})
      self.css({"position": 'fixed', 'bottom': '0px', 'right': '5px'})
      self.add_span("%s..." % text, position="after", css={"width": 'auto'})
    else:
      self.add_span("%s..." % text, position="after", css={"width": '100%', "margin": "5px"})

  def fixed(self, css=None, icon_css=None):
    """
    Description:
    ------------
    Set css attributes of the loading div to be fixed
    This can be done directly in options in the component constructor options={"fixed": True}

    Attributes:
    ----------
    :param css: Dictionary with the css attributes
    :param icon_css: Dictionary with the CSS attributes

    :return: self to allow the chains
    """
    dflt_css = {"position": 'fixed', 'bottom': '5px', 'right': '5px'}
    if css is not None:
      dflt_css.update(css)
    dflt_css_icon = {"margin-right": '5px', "font-size": 'inherit'}
    if dflt_css_icon is not None:
      dflt_css_icon.update(icon_css)
    self.icon.css(dflt_css_icon)
    self.css(dflt_css)
    return self

  def __str__(self):
    return '<div %s></div>' % (self.get_attrs(pyClassNames=self.style.get_classes()))


class HtmlJson(Html.Html):
  name = 'Pretty Json'
  requirements = ('json-formatter', )

  def __init__(self, report, data, width, height, options, profile):
    super(HtmlJson, self).__init__(report, data, profile=profile, css_attrs={"height": height, width: "width"})
    self.__options = OptJsonFormatter.OptionsJsonFmt(self, options)

  @property
  def jsonId(self):
    """
    Return the Javascript variable of the json object
    """
    return "%s_obj" % self.htmlCode

  @property
  def _js__builder__(self):
    return '''
      window[ htmlObj.id + '_obj'] = new JSONFormatter(data, options.open, options.opts); htmlObj.innerHTML = '';
      htmlObj.appendChild(window[ htmlObj.id + '_obj'].render());
      '''

  @property
  def options(self):
    """

    :rtype: OptJsonFormatter.OptionsJsonFmt
    """
    return self.__options

  @property
  def js(self):
    """
    Return the Javascript internal object

    :return: A Javascript object

    :rtype: JsJsonFormatter.Json
    """
    if self._js is None:
      self._js = JsJsonFormatter.Json(self._report, varName=self.jsonId, setVar=False, parent=self)
    return self._js

  def __str__(self):
    self._report._props.setdefault('js', {}).setdefault("builders", []).append(self.refresh())
    return '<div %s></div>' % (self.get_attrs(pyClassNames=self.style.get_classes()))




from epyk.core.css.styles.classes import CssStyle
from epyk.core.html import Defaults as Defaults_html
from epyk.core.css import Defaults as Defaults_css


class CssSelectStyle(CssStyle.Style):
  _attrs = {'padding-top': '2px'}
  _focus = {'outline': 0, 'border': 'none', 'box-shadow': 'none'}

  def customize(self):
    self.css({"background": self.rptObj.theme.greys[0], "color": self.rptObj.theme.greys[-1],
              'font-family': Defaults_css.Font.family, 'line-height': '%spx' % Defaults_html.LINE_HEIGHT,
              'font-size': '%spx' % Defaults_css.Font.size, #'min-width': '%spx' % Defaults_html.INPUTS_MIN_WIDTH
              })
    self.css({'display': 'inline-block', 'margin': 0}, important=True)


class CssSelectButton(CssStyle.Style):
  _attrs = {'padding': '0 5px', 'outline': 'none', 'border-color': 'none', 'box-shadow': 'none'}
  _focus = {'outline': 'none', 'border-color': 'none', 'box-shadow': 'none'}

  _selectors = {"child": '.btn'}

  def customize(self):
    self.css({"background-color": self.rptObj.theme.colors[0], "border": 'none', 'color': self.rptObj.theme.colors[-1]})


class CssSelectSearchBoxInput(CssStyle.Style):
  _attrs = {"outline": 0, "margin-bottom": "5px"}
  _focus = {'outline': 0}

  classname = 'bs-searchbox'
  _selectors = {"child": 'input.form-control'}

  def customize(self):
    self.css({"border-color": self.rptObj.theme.colors[5], "height": "%spx" % Defaults_html.LINE_HEIGHT})
    self.focus.css({"box-shadow": "0 0 0 0.2em %s" % self.rptObj.theme.colors[0]})


class CssSelectToggle(CssStyle.Style):
  classname = "dropdown-toggle"

  def customize(self):
    self.css({"background-color": self.rptObj.theme.colors[0], 'font-size': '%s%s' % (Defaults_css.Font.size, Defaults_css.Font.unit)}, important=True)


class CssSelectOption(CssStyle.Style):
  classname = "dropdown-menu"

  def customize(self):
    self.css({"background": self.rptObj.theme.greys[-1], 'font-size': '%s%s' % (Defaults_css.Font.size, Defaults_css.Font.unit)})


class CssSelectOptionItems(CssStyle.Style):
  _attrs = {"margin": 0, "padding": "0 5px"}
  classname = "dropdown-item"
  _focus = {'outline': 0}

  def customize(self):
    self.active.css({"background-color": self.rptObj.theme.success[1]})


class CssSelectOptionActive(CssStyle.Style):
  classname = "active"

  def customize(self):
    self.css({'background': self.rptObj.theme.success[0], "color": self.rptObj.theme.success[1]}, important=True)


class CssSelectOptionSelected(CssStyle.Style):
  classname = "selected"

  def customize(self):
    self.css({'background': self.rptObj.theme.success[0], "color": self.rptObj.theme.success[1]}, important=True)


class CssSelectFilterOption(CssStyle.Style):
  _attrs = {'text-align': 'center !IMPORTANT'}
  classname = "filter-option"


class CssSelectOutline(CssStyle.Style):
  _focus = {'outline': '0 !important'}
  classnames = ["bootstrap-select", "dropdown-toggle"]


class CssSelectStatus(CssStyle.Style):
  classnames = ["dropdown-menu", "status"]

  def customize(self):
    self.css({"background-color": self.rptObj.theme.greys[0], 'font-size': '%s%s' % (Defaults_css.Font.size, Defaults_css.Font.unit)})



"""
CSS Style module for the Chart components
"""

from epyk.core.css.styles.classes import CssStyle


class CssDivChart(CssStyle.Style):
  """
  CSS class for the Chart container
  """
  _attrs = {'margin': '0 0 5px 0', 'outline': 'none'}

  def customize(self):
    self.css({'border': "1px solid %s" % self.rptObj.theme.greys[3]})
    self.hover.css({'border': "1px solid %s" % self.rptObj.theme.colors[1]})


# --------------------------------------------------------------------------------------------------------------
#
#                                   BILLBOARD SECTION
# --------------------------------------------------------------------------------------------------------------
class CssBillboardTitle(CssStyle.Style):
  classname = "bb-title"

  def customize(self):
    self.css({'fill': self.rptObj.theme.greys[-1]}, important=True)


class CssBillboardLegend(CssStyle.Style):
  classname = "bb text"

  def customize(self):
    self.css({'fill': self.rptObj.theme.greys[-1]}, important=True)


class CssBillboardAxis(CssStyle.Style):
  _selectors = {'child': ".bb-axis line, .bb-axis-x line, .bb-axis .domain, .bb-axis-x .domain"}

  def customize(self):
    self.css({'stroke': self.rptObj.theme.greys[-1]})


class CssBillboardXAxis(CssStyle.Style):
  classname = "bb-axis-x text"

  def customize(self):
    self.css({'fill': self.rptObj.theme.greys[-1]}, important=True)


class CssBillboardYAxis(CssStyle.Style):
  classname = "bb-axis-y text"

  def customize(self):
    self.css({'fill': self.rptObj.theme.greys[-1]}, important=True)


# --------------------------------------------------------------------------------------------------------------
#
#                                   C3 SECTION
# --------------------------------------------------------------------------------------------------------------
class CssC3Title(CssStyle.Style):
  classname = "c3-title"

  def customize(self):
    self.css({'fill': self.rptObj.theme.greys[-1]}, important=True)


class CssC3Legend(CssStyle.Style):
  classname = "c3 text"

  def customize(self):
    self.css({'fill': self.rptObj.theme.greys[-1]}, important=True)


class CssC3Axis(CssStyle.Style):
  _selectors = {'child': '.c3-axis line, .c3-axis .domain'}

  def customize(self):
    self.css({'stroke': self.rptObj.theme.greys[-1]})


class CssC3XAxis(CssStyle.Style):
  classname = "c3-axis-x"
  _selectors = {'child': "text"}

  def customize(self):
    self.css({'fill': self.rptObj.theme.greys[-1]}, important=True)


class CssC3YAxis(CssStyle.Style):
  classname = "c3-axis-y"
  _selectors = {'child': "text"}

  def customize(self):
    self.css({'fill': self.rptObj.theme.greys[-1]}, important=True)


# --------------------------------------------------------------------------------------------------------------
#
#                                   NVD3 SECTION
# --------------------------------------------------------------------------------------------------------------
class CssNVD3Axis(CssStyle.Style):
  _selectors = {'child': '.nvd3 .nv-axis g path.domain'}

  def customize(self):
    self.css({'stroke': self.rptObj.theme.greys[-1], 'stroke-opacity': 1})


class CssNVD3AxisLabel(CssStyle.Style):
  classnames = ['nvd3', 'nv-axis']

  def customize(self):
    self.css({'fill': self.rptObj.theme.greys[-1]})


class CssNVD3AxisLegend(CssStyle.Style):
  _selectors = {'child': 'svg text'}

  def customize(self):
    self.css({'fill': self.rptObj.theme.greys[-1]})


class CssNVD3HideGrid(CssStyle.Style):
  classnames = ['nvd3', 'tick']
  _selectors = {'child': 'line'}

  def customize(self):
    self.css({'display': 'none'})


# Bootstrap issue
# https://stackoverflow.com/questions/18894820/jquery-sparklines-and-twitter-bootstrap-3-tooltip-style-overrides
class CssSparklines(CssStyle.Style):
  classname = "jqstooltip"
  _attrs = {"-webkit-box-sizing": "content-box", "-moz-box-sizing": "content-box", "box-sizing": "content-box"}


# --------------------------------------------------------------------------------------------------------------
#
#                                   VIS SECTION
# --------------------------------------------------------------------------------------------------------------
class CssVisItems(CssStyle.Style):
  classname = "vis-item"
  _attrs = {'box-shadow': '5px 5px 20px rgba(128,128,128, 0.5)'}

  def customize(self):
    self.css({'border-color': self.rptObj.theme.colors[-1], 'background-color': self.rptObj.theme.colors[2]}, change=False)


class CssVisItemsOverlow(CssStyle.Style):
  classname = "vis-item-overflow"
  _attrs = {}

  def customize(self):
    self.css({'border': "1px solid %s" % self.rptObj.theme.success[1], 'background': self.rptObj.theme.colors[0]})





from epyk.core.js import JsUtils

from epyk.core.js.primitives import JsArray
from epyk.core.js.primitives import JsBoolean
from epyk.core.js.primitives import JsObject
from epyk.core.js.primitives import JsString
from epyk.core.js.primitives import JsNumber


class Event(object):

  def getEvent(self, varName):
    return JsObject.JsObject.get(varName)

  def createEvent(self, varName, type='Event'):
    """
    Description:
    ------------

    Related Pages:

			https://developer.mozilla.org/fr/docs/Web/API/Document/createEvent

    Attributes:
    ----------
    :param type:
    :param varName:
    """
    return "var %s = document.createEvent('%s')" % (varName, type)

  def initEvent(self, name, varName=None, bubbles=True, cancelable=True):
    """
    Description:
    ------------
    The Event.initEvent() method is used to initialize the value of an event created using Document.createEvent().

    Related Pages:

			https://developer.mozilla.org/en-US/docs/Web/API/Event/initEvent

    Attributes:
    ----------
    :param name:
    :param bubbles:
    :param cancelable:
    """
    name = JsUtils.jsConvertData(name, None)
    bubbles = JsUtils.jsConvertData(bubbles, None)
    cancelable = JsUtils.jsConvertData(cancelable, None)
    varName = varName or "document.createEvent('Event')"
    return JsObject.JsObject.get("%s.initEvent(%s, %s, %s)" % (varName, name, bubbles, cancelable))

  def cancelBubble(self):
    """
    Description:
    ------------
    The cancelBubble() method prevents the event-flow from bubbling up to parent elements.

    Related Pages:

			https://www.w3schools.com/jsref/event_cancelbubble.asp
    """
    return JsString.JsString("event.cancelBubble = true")

  def target(self):
    """
    Description:
    ------------
    Returns the element that triggered the event

    Related Pages:

			https://www.w3schools.com/jsref/event_target.asp
    """
    return JsString.JsString("event.target")

  @property
  def dataTransfer(self):
    """
    Description:
    ------------
    The DataTransfer object is used to hold the data that is being dragged during a drag and drop operation.
    It may hold one or more data items, each of one or more data types. For more information about drag and drop, see HTML Drag and Drop API.

    This object is available from the dataTransfer property of all drag events.

    Related Pages:

			https://developer.mozilla.org/fr/docs/Web/API/DataTransfer
    """#
    from epyk.core.js.objects.JsData import JsDataTransfer

    return JsDataTransfer("event.dataTransfer")

  @property
  def clipboardData(self):
    """
    Description:
    ------------
    The ClipboardEvent.clipboardData property holds a DataTransfer object, which can be used:

      - to specify what data should be put into the clipboard from the cut and copy event handlers, typically with a setData(format, data) call;
      - to obtain the data to be pasted from the paste event handler, typically with a getData(format) call.

    Related Pages:

      https://developer.mozilla.org/en-US/docs/Web/API/ClipboardEvent/clipboardData
    """
    from epyk.core.js.objects.JsData import JsClipboardData

    return JsClipboardData("event.clipboardData")

  @property
  def timeStamp(self):
    """
    Description:
    ------------
    Returns the time (in milliseconds relative to the epoch) at which the event was created

    Related Pages:

			https://www.w3schools.com/jsref/event_timestamp.asp
    """
    return JsString.JsString("event.timeStamp", isPyData=False)

  @property
  def defaultPrevented(self):
    """
    Description:
    ------------
    The defaultPrevented event property checks whether the preventDefault() method was called for the event.

    Related Pages:

			https://www.w3schools.com/jsref/event_defaultprevented.asp
    """
    return JsString.JsString("event.defaultPrevented", isPyData=False)

  def preventDefault(self):
    """
    Description:
    ------------
    Cancels the event if it is cancelable, meaning that the default action that belongs to the event will not occur

    Related Pages:

			https://www.w3schools.com/jsref/event_preventdefault.asp
    """
    return JsString.JsString("event.preventDefault()", isPyData=False)

  def stopImmediatePropagation(self):
    """
    Description:
    ------------
    The stopImmediatePropagation() method of the Event interface prevents other listeners of the same event from being called.

    Related Pages:

			https://developer.mozilla.org/en-US/docs/Web/API/Event/stopImmediatePropagation
    """
    return JsString.JsString("event.stopImmediatePropagation()", isPyData=False)

  def stopPropagation(self):
    """
    Description:
    ------------
    Prevents further propagation of an event during event flow

    Related Pages:

			https://www.w3schools.com/jsref/event_stoppropagation.asp
    """
    return JsString.JsString("event.stopPropagation()", isPyData=False)

  def toStr(self):
    return "event"


class UIEvent(Event):

  @property
  def detail(self):
    """
    Description:
    ------------
    The detail property returns a number with details about the event.

    Related Pages:

			https://www.w3schools.com/jsref/event_detail.asp
    """
    return JsNumber.JsNumber("event.detail", isPyData=False)

  @property
  def view(self):
    """
    Description:
    ------------
    The view event property returns a reference to the Window object where the event occured.

    Related Pages:

			https://www.w3schools.com/jsref/event_view.asp
    """
    from epyk.core.js.JsWindow import JsWindow

    return JsWindow()


class KeyboardEvent(UIEvent):

  @property
  def altKey(self):
    """
    Description:
    ------------
    The altKey property returns a Boolean value that indicates whether or not the "ALT" key was pressed when a key event was triggered.

    Related Pages:

			https://www.w3schools.com/jsref/event_key_altkey.asp
    """
    return JsBoolean.JsBoolean.get(varName="event.altKey")

  @property
  def charCode(self):
    """
    Description:
    ------------
    The charCode property returns the Unicode character code of the key that triggered the onkeypress event.

    Related Pages:

			https://www.w3schools.com/jsref/event_key_charcode.asp
    """
    return JsString.JsString.get(varName="event.charCode")

  @property
  @JsUtils.incompatibleBrowser(["Internet Explorer"])
  def code(self):
    """
    Description:
    ------------
    The code property returns the key that triggered the event.

    Related Pages:

			https://www.w3schools.com/jsref/event_key_code.asp
    """
    return JsString.JsString("event.code", isPyData=False)

  @property
  def ctrlKey(self):
    """
    Description:
    ------------
    The ctrlKey property returns a Boolean value that indicates whether or not the "CTRL" key was pressed when a key event was triggered.

    Related Pages:

			https://www.w3schools.com/jsref/event_key_ctrlkey.asp
    """
    return JsBoolean.JsBoolean("event.ctrlKey", isPyData=False)

  @property
  @JsUtils.incompatibleBrowser(["Safari"])
  def key(self):
    """
    Description:
    ------------
    The getModifierState() method returns true if the specified modifier key was pressed, or activated.

    Related Pages:

			https://www.w3schools.com/jsref/event_key_key.asp
    """
    return JsString.JsString("event.key", isPyData=False)

  @property
  def keyCode(self):
    """
    Description:
    ------------
    The keyCode property returns the Unicode character code of the key that triggered the onkeypress event, or the Unicode key code of the key that triggered the onkeydown or onkeyup event.

    Related Pages:

			https://www.w3schools.com/jsref/event_key_keycode.asp
    """
    return JsString.JsString("event.keyCode", isPyData=False)

  @property
  @JsUtils.incompatibleBrowser(["Safari"])
  def location(self):
    """
    Description:
    ------------
    The location property returns a number that indicates the location of a key on the keyboard or device.

    Related Pages:

			https://www.w3schools.com/jsref/event_key_location.asp
    """
    return JsNumber.JsNumber("event.location", isPyData=False)

  @property
  def metaKey(self):
    """
    Description:
    ------------
    The metaKey property returns a Boolean value that indicates whether or not the "META" key was pressed when a key event was triggered.

    Related Pages:

			https://www.w3schools.com/jsref/event_key_metakey.asp
    """
    return JsString.JsString("event.metaKey", isPyData=False)

  @property
  def shiftKey(self):
    """
    Description:
    ------------
    The shiftKey property returns a Boolean value that indicates whether or not the "SHIFT" key was pressed when a key event was triggered.

    Related Pages:

			https://www.w3schools.com/jsref/event_key_shiftkey.asp
    """
    return JsBoolean.JsBoolean.get(varName="event.shiftKey")

  @property
  def which(self):
    """
    Description:
    ------------
    The which property returns the Unicode character code of the key that triggered the onkeypress event, or the Unicode key code of the key that triggered the onkeydown or onkeyup event.

    Related Pages:

			https://www.w3schools.com/jsref/event_key_which.asp
    """
    return JsNumber.JsNumber("event.which", isPyData=False)


class MouseEvent(UIEvent):

  @property
  def altKey(self):
    """
    Description:
    ------------
    The altKey property returns a Boolean value that indicates whether or not the "ALT" key was pressed when a key event was triggered.

    Related Pages:

			https://www.w3schools.com/jsref/event_altkey.asp
    """
    return JsBoolean.JsBoolean.get(varName="event.altKey")

  @property
  def button(self):
    """
    The button property returns a number that indicates which mouse button was pressed when a mouse event was triggered.

    Related Pages:

			https://www.w3schools.com/jsref/event_button.asp
    """
    return JsNumber.JsNumber("event.button", isPyData=False)

  @property
  @JsUtils.incompatibleBrowser(["Safari"])
  def buttons(self):
    """
    The buttons property returns a number that indicates which mouse button or mouse buttons were pressed when a mouse event was triggered.

    Related Pages:

			https://www.w3schools.com/jsref/event_buttons.asp
    """
    return JsNumber.JsNumber("event.buttons", isPyData=False)

  @property
  def isTrusted(self):
    """

    :return:
    """
    return JsBoolean.JsBoolean.get(varName="event.isTrusted")

  @property
  def clientX(self):
    """
    Description:
    ------------
    Returns the horizontal coordinate of the mouse pointer, relative to the current window, when the mouse event was triggered

    Related Pages:

			https://www.w3schools.com/jsref/event_clientx.asp
    """
    return JsNumber.JsNumber.get(varName="event.clientX")

  @property
  def clientY(self):
    """
    Description:
    ------------
    The clientY property returns the vertical coordinate (according to the client area) of the mouse pointer when a mouse event was triggered.

    Related Pages:

			https://www.w3schools.com/jsref/event_clienty.asp
    """
    return JsNumber.JsNumber.get(varName="event.clientY")

  @property
  def pageX(self):
    """

    :return:
    """
    return JsNumber.JsNumber.get(varName="event.pageX")

  @property
  def pageY(self):
    """

    :return:
    """
    return JsNumber.JsNumber.get(varName="event.pageY")

  @property
  def offsetX(self):
    """
    Description:
    ------------
    Returns the horizontal coordinate of the mouse pointer relative to the position of the edge of the target element
    """
    return JsNumber.JsNumber.get(varName="event.offsetX")

  @property
  def offsetY(self):
    """
    Description:
    ------------
    Returns the horizontal coordinate of the mouse pointer relative to the position of the edge of the target element
    """
    return JsNumber.JsNumber.get(varName="event.offsetY")

  def getField(self, fieldName):
    return JsObject.JsObject.get("event.%s" % fieldName)

  @property
  def metaKey(self):
    """
    The metaKey property returns a Boolean value that indicates whether or not the "META" key was pressed when a key event was triggered.

    https://www.w3schools.com/jsref/event_metakey.asp
    """
    return JsString.JsString("event.metaKey", isPyData=False)

  @property
  def shiftKey(self):
    """
    The shiftKey property returns a Boolean value that indicates whether or not the "SHIFT" key was pressed when a mouse event was triggered.

    https://www.w3schools.com/jsref/event_shiftkey.asp
    """
    return JsBoolean.JsBoolean.get(varName="event.shiftKey")

  @property
  def ctrlKey(self):
    """
    The ctrlKey property returns a Boolean value that indicates whether or not the "CTRL" key was pressed when a mouse event was triggered.

    https://www.w3schools.com/jsref/event_ctrlkey.asp
    """
    return JsBoolean.JsBoolean("event.ctrlKey", isPyData=False)

  @property
  def which(self):
    """
    The which property returns a number that indicates which mouse button was pressed when a mouse event was triggered.

    https://www.w3schools.com/jsref/event_which.asp
    """
    return JsNumber.JsNumber("event.which", isPyData=False)

  def toStr(self):
    return JsObject.JsObject.get("event")


class TouchEvent(UIEvent):

  @property
  def altKey(self):
    """
    Description:
    ------------
    The altKey property returns a Boolean value that indicates whether or not the "ALT" key was pressed when a touch event was triggered.

    Related Pages:

			https://www.w3schools.com/jsref/event_touch_altkey.asp
    """
    return JsBoolean.JsBoolean.get(varName="event.altKey")

  @property
  def ctrlKey(self):
    """
    Description:
    ------------
    The ctrlKey property returns a Boolean value that indicates whether or not the "CTRL" key was pressed when a touch event was triggered.

    Related Pages:

			https://www.w3schools.com/jsref/event_touch_ctrlkey.asp
    """
    return JsBoolean.JsBoolean("event.ctrlKey", isPyData=False)

  @property
  def metaKey(self):
    """
    Description:
    ------------
    The metaKey property returns a Boolean value that indicates whether or not the "META" key was pressed when a touch event was triggered.

    Related Pages:

			https://www.w3schools.com/jsref/event_touch_metakey.asp
    """
    return JsString.JsString("event.metaKey", isPyData=False)

  @property
  def shiftKey(self):
    """
    Description:
    ------------
    The shiftKey property returns a Boolean value that indicates whether or not the "SHIFT" key was pressed when a touch event was triggered.

    Related Pages:

			https://www.w3schools.com/jsref/event_touch_shiftkey.asp
    """
    return JsBoolean.JsBoolean.get(varName="event.shiftKey")

  @property
  def targetTouches(self):
    """
    Description:
    ------------
    The targetTouches property returns an array of Touch objects, one for each finger that is touching the current target element.

    Related Pages:

			https://www.w3schools.com/jsref/event_touch_targettouches.asp
    """
    return JsArray.JsArray("event.shiftKey", isPyData=False)

  @property
  def touches(self):
    """
    Description:
    ------------
    The touches property returns an array of Touch objects, one for each finger that is currently touching the surface.

    Related Pages:

			https://www.w3schools.com/jsref/event_touch_touches.asp
    """
    return JsArray.JsArray("event.shiftKey", isPyData=False)

  def ontouchcancel(self):
    pass

  def ontouchend(self):
    pass

  def ontouchmove(self):
    pass

  def ontouchstart(self):
    pass




from epyk.core.css import Defaults_css
from epyk.core.css.styles.attributes import Attrs


class NavBar(Attrs):
  def __init__(self, htmlObj):
    super(NavBar, self).__init__(htmlObj)
    self.font_size = Defaults_css.font(5)
    self.display = 'block'
    self.margin = 0
    self.vertical_align = 'top'
    self.left = 0
    self.padding = "0 2px 0 2px"
    self.position = "fixed"
    self.background_color = htmlObj._report.theme.greys[0]
    self.border_bottom = "1px solid %s" % htmlObj._report.theme.greys[4]
    self.top = 0
    self.z_index = 10

class Footer(Attrs):
  def __init__(self, htmlObj):
    super(Footer, self).__init__(htmlObj)
    self.display = 'block'
    self.margin = 0
    self.vertical_align = 'bottom'
    self.left = 0
    self.padding = "0 2px 0 2px"
    self.position = "fixed"
    self.background_color = htmlObj._report.theme.greys[1]
    self.border_top = "1px solid %s" % htmlObj._report.theme.greys[4]
    self.bottom = 0
    self.z_index = 10



"""
Module dedicated to perform the data transformation for the D3 charts
"""

from epyk.core.js import JsUtils


class JsChartD3Links(object):
  def __init__(self, data, js_src, data_schema=None, profile=False):
    self._js_src, self._data_schema, self.profile, self._data = js_src, data_schema, profile, data

  def __register_records_fnc(self, fnc_name, fnc_def, fnc_pmts=None, profile=False):
    """
    This function will attach to the report object only the javascript functions used during the report

    :param fnc_name: A String with the Javascript function name to be defined
    :param fnc_def: A String with the Javascript function content
    :param fnc_pmts: A list of parameters
    :param profile: A boolean flag to activate the framework profiling

    :return:
    """
    fnc_pmts = ["data"] + (fnc_pmts or [])
    if not fnc_name in self._js_src.get('js', {}).get('functions', {}):
      if profile:
        content = "var result = []; %s;return result" % JsUtils.cleanFncs(fnc_def)
      else:
        content = "var result = []; %s;return result" % JsUtils.cleanFncs(fnc_def)
      self._js_src.setdefault('js', {}).setdefault('functions', {})[fnc_name] = {'content': content, 'pmt': fnc_pmts}

  def line(self):
    pass


class JsD3(object):
  """

  """
  alias = "D3"
  value = ''


class JsD3Bubble(object):
  alias = "D3"
  chartTypes = ['gravity']
  params = ("seriesNames", "xAxis")
  value = '''
    nbSeries = seriesNames.length;
    var temp = {}; var result = [];
    data.forEach(function(rec){
      for (var key in rec){
        if(key != xAxis && rec[key] != 0 && seriesNames.includes(key))
        { 
          result.push({'Name': rec[xAxis], 'Count': rec[key], 'Category': key, 'nbSeries': nbSeries, 'xAxis': xAxis, 'seriesNames': seriesNames});
        } 
      }
    });
    '''




from epyk.core.html import Defaults


class Component(object):

  def __init__(self, html):
    self._html = html
    self.name = html.__class__.__name__
    self.folder = self.name.lower()

  def rename(self, name):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param name:
    """
    self.name = name.capitalize()
    self.folder = self.name.lower()
    return self

  @property
  def tag(self):
    """

    """
    return "%s-%s" % (Defaults.COMP_PREFIX, self.folder)

  def ts(self, name=None):
    """
    Description:
    -----------
    Component export for an Angular app.
    This will allow to build apps directly from reports using each underlying components available in the framework.

    Attributes:
    ----------
    :param name: Optional. The component name. Can be set to define test components
    """
    if name is not None:
      self.rename(name)
    self._out_mode = "angular"
    html = self._html.html()
    return {'folder': self.folder, 'class': "Epyk%sComponent" % self.name, 'externalVars': '', 'css': self._html.attr['css'],
            'htmlTag': self.tag, 'build': self._html._js__builder__.strip(), 'html': html, 'options': self._html._jsStyles}





class Selector(object):

  def __init__(self, component=None):
    self._js = []
    if component is not None:
      self._js.append("#%s" % component.htmlCode)

  def with_attribute(self, name, value=None, startswith=False, containing=False, endswith=False):
    """
    Description:
    -----------
    Specify HTML attributes to select

    Related Pages:

        https://www.w3schools.com/cssref/sel_attribute_value.asp

    Attributes:
    ----------
    :param name: String. The attribute name
    :param value: String. The attribute value
    """
    if value is None:
      self._js.append("[%s]" % name)
    elif startswith:
      self._js.append("[%s|=%s]" % (name, value))
    elif containing:
      self._js.append("[%s~=%s]" % (name, value))
    elif endswith:
      self._js.append("[%s$=%s]" % (name, value))
    else:
      self._js.append("[%s=%s]" % (name, value))
    return self

  def with_next_element(self, element):
    """
    Description:
    -----------
    The element>element selector is used to select elements with a specific parent.

    Related Pages:

        https://www.w3schools.com/cssref/sel_element_gt.asp

    Attributes:
    ----------
    :param element: String. The HTML type (tag)
    """
    self._js.append(" + %s" % element)
    return self

  def with_child_element(self, element):
    """
    Description:
    -----------
    The element>element selector is used to select elements with a specific parent.

    Related Pages:

        https://www.w3schools.com/cssref/sel_element_gt.asp

    Attributes:
    ----------
    :param element: String. The HTML type (tag)
    """
    self._js.append(" > %s" % element)
    return self

  def elements(self, tags):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param tags: List. The HTML types (tags)
    """
    self._js.append(" %s" % ",".join(tags))
    return self

  def state(self, value):
    """
    Description:
    -----------
    Selects a specific status for the item

    usage:

        self.state("disabled")

    Related Pages:

			https://www.w3schools.com/cssref/css_selectors.asp

    Attributes:
    ----------
    :params value: String. The state value
    """
    self._js.append(":%s" % value)
    return self

  def focus(self):
    """
    Description:
    -----------
    Selects the input element which has focus

    Related Pages:

			https://www.w3schools.com/cssref/sel_focus.asp
    """
    self._js.append(":focus")
    return self

  def hover(self):
    """
    Description:
    -----------
    Selects links on mouse over

    Related Pages:

			https://www.w3schools.com/cssref/sel_hover.asp
    """
    self._js.append(":hover")
    return self

  def element(self, tag):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param tag: String. The HTML tag
    """
    self._js.append(" %s" % tag)
    return self

  def with_htmlcode(self, htmlCode):
    """
    Description:
    -----------

    Attributes:
    ----------
    :param htmlCode:
    """
    self._js.append("#%s" % htmlCode)
    return self

  def with_class(self, class_name):
    """
    Description:
    -----------
    Specify the class to use as a filter

    https://www.w3schools.com/cssref/css_selectors.asp

    Attributes:
    ----------
    :param class_name: String. The CSS Class name
    """
    self._js.append(".%s" % class_name)
    return self

  def excluding(self, selector):
    """
    Description:
    -----------
    Specify the element to exclude from the selection

    https://www.w3schools.com/cssref/css_selectors.asp

    Attributes:
    ----------
    :param selector: String or Selector. The id to exclude
    """
    if hasattr(selector, 'htmlCode'):
      selector = "[id='%s']" % selector.htmlCode
    self._js.append(":not(%s)" % selector)
    return self

  def first_child(self):
    """
    Description:
    -----------
    Selects every <> element that is the first child of its parent
    """
    self._js.append(":first-child")
    return self

  def last_child(self):
    """
    Description:
    -----------
    Selects every <> element that is the last child of its parent
    """
    self._js.append(":last-child")
    return self

  def first_letter(self):
    """
    Description:
    -----------
    Selects the first letter of every <> element
    """
    self._js.append("::first-letter")
    return self

  def first_of_type(self, type):
    """
    Description:
    -----------
    Selects every <> element that is the first <> element of its parent

    Attributes:
    ----------
    :param type: String. The HTML type (tag)
    """
    self._js.append("%s:first-of-type" % type)
    return self

  def last_of_type(self, type):
    """
    Description:
    -----------
    Selects every <> element that is the last <> element of its parent

    Attributes:
    ----------
    :param type: String. The HTML type (tag)
    """
    self._js.append("%s:last-of-type" % type)
    return self

  def nth_child(self, i):
    """
    Description:
    -----------
    Selects every <> element that is the second child of its parent

    Attributes:
    ----------
    :param i: Integer. The index (starting from 1)
    """
    self._js.append(":nth-child(%s)" % i)
    return self

  def nth_last_child(self, i):
    """
    Description:
    -----------
    Selects every <> element that is the second child of its parent, counting from the last child

    Attributes:
    ----------
    :param i: Integer. The index (starting from 1)
    """
    self._js.append(":nth-last-child(%s)" % i)
    return self

  def nth_last_of_type(self, type, i):
    """
    Description:
    -----------
    Selects every <p> element that is the second <> element of its parent, counting from the last child

    Attributes:
    ----------
    :param type: String. The HTML type (tag)
    :param i:  Integer. The index (starting from 1)
    """
    self._js.append("%s:nth-last-of-type(%s)" % (type, i))
    return self

  def nth_of_type(self, type, i):
    """
    Description:
    -----------
    Selects every <p> element that is the only <> element of its parent

    Attributes:
    ----------
    :param type: String. The HTML type (tag)
    :param i:  Integer. The index (starting from 1)
    """
    self._js.append("%s:nth-of-type(%s)" % (type, i))
    return self

  def only_of_type(self, type):
    """
    Description:
    -----------
    The :only-of-type selector matches every element that is the only child of its type, of its parent.

    Attributes:
    ----------
    :param type: String. The HTML type (tag)
    """
    self._js.append("%s:only-of-type" % type)
    return self

  def only_child(self):
    """
    Description:
    -----------
    The :only-child selector matches every element that is the only child of its parent.
    """
    self._js.append(":only-child")
    return self

  def toStr(self):
    from epyk.core.js import JsUtils

    return JsUtils.jsConvertData("".join(self._js), None)

  def __str__(self):
    return str(self.toStr())



"""
Javascript Interface to the Datatable Module

Related Pages:

		https://datatables.net/reference/api/
"""

from epyk.core.js import JsUtils

from epyk.core.js.primitives import JsObjects
from epyk.core.js.objects import JsNodeDom
from epyk.core.js.packages import JsQuery
from epyk.core.js.packages import JsPackage


class SelectAPI(JsPackage):
  """

  """

  def blurable(self):
    """
    Get the blurable state for the table

    Related Pages:

			https://datatables.net/reference/api/select.blurable()

    :return:
    """
    return JsObjects.JsBoolean.JsBoolean("%s.blurable()" % self._selector)

  def info(self, jsFlag=None):
    """
    Get / set the information summary display state.

    Related Pages:

			https://datatables.net/reference/api/select.info()

    :param jsFlag: Value to set for the information summary display state - true to enable, false to disable.
    :return:
    """
    if jsFlag is None:
      return JsObjects.JsBoolean.JsBoolean("%s.info()" % self._selector)

    return JsObjects.JsObject.JsObject("%s.info(%s)" % (self._selector, JsUtils.jsConvertData(jsFlag, None)))

  def items(self):
    """
    Get / set the items that Select will select based on user interaction (rows, columns or cells).

    Related Pages:

			https://datatables.net/reference/api/select.items()

    :return:
    """
    return JsObjects.JsString.JsString("%s.items()" % self._selector)

  def selector(self):
    """
    Get the current item selector string applied to the table.

    Related Pages:

			https://datatables.net/reference/api/select.selector()

    :return:
    """
    return JsObjects.JsString.JsString("%s.selector()" % self._selector)

  def style(self):
    """
    Get / set the style by which the end user can select items in the table.

    Related Pages:

			https://datatables.net/reference/api/select.style()

    :return:
    """
    return JsObjects.JsString.JsString("%s.style()" % self._selector)


class CellAPI(JsPackage):
  lib_alias = {'js': "datatables", 'css': 'datatables'}
  lib_selector = 'cell'
  lib_set_var = False

  def deselect(self):
    """
    Deselect a single cell

    Related Pages:

			https://datatables.net/reference/api/column().deselect()

    :return: DataTables API instance for chaining
    """
    return self.fnc("deselect()")

  def select(self):
    """
    Select a single cell

    Related Pages:

			https://datatables.net/reference/api/cell().select()

    :return: DataTables API instance for chaining
    """
    return self.fnc("select()")

  def render(self):
    """
    Get rendered data for a cell

    Related Pages:

			https://datatables.net/reference/api/cell().render()

    :return: DataTables API instance for chaining
    """
    return self.fnc("render()")

  def node(self):
    """
    Get the DOM element for the selected cell

    Related Pages:

			https://datatables.net/reference/api/cell().node()

    :return: DataTables API instance for chaining
    """
    self.fnc("node()")

  def jquery_nodes(self):
    """
    Get the cell nodes for the selected column.

    Related Pages:

			https://datatables.net/reference/api/column().nodes()

    :return:
    """
    self.nodes()
    self._js.append("nodes().to$()")
    return JsQuery.JQuery(jqId=self.toStr())

  def invalidate(self):
    """
    Invalidate the data held in DataTables for the selected cells

    Related Pages:

			https://datatables.net/reference/api/cell().invalidate()

    :return: DataTables API instance for chaining
    """
    return self.fnc("invalidate()")

  def index(self):
    """
    Get index information about the selected cell

    Related Pages:

			https://datatables.net/reference/api/cell().index()

    :return:
    """
    return JsObjects.JsNumber.JsNumber("%s.index()" % self.getStr())

  def cache(self):
    """
    Get cached data of the cache type specified

    Related Pages:

			https://datatables.net/reference/api/cell().cache()

    :return:
    """

  def data(self):
    """
    Get / set data for the selected cell.

    Related Pages:

			https://datatables.net/reference/api/cell().data()

    :return:
    """
    return self.fnc("data()")

  def focus(self):
    """
    Focus on a cell.

    Related Pages:

			https://datatables.net/reference/api/cell().focus()

    :return:
    """
    return self.fnc("focus()")

  def blur(self):
    """
    Blur focus from the table.

    Related Pages:

			https://datatables.net/reference/api/cell.blur()

    :return:
    """
    return self.fnc("blur()")


class ColumnAPI(JsPackage):
  lib_alias = {'js': "datatables", 'css': 'datatables'}
  lib_selector = 'column'
  lib_set_var = False

  def deselect(self):
    """
    Deselect a single column

    Related Pages:

			https://datatables.net/reference/api/column().deselect()

    :return: DataTables API instance for chaining
    """
    return self.fnc("deselect()")

  def select(self):
    """
    Select a single column

    Related Pages:

			https://datatables.net/reference/api/column().select()

    :return: DataTables API instance for chaining
    """
    self.fnc("select()")

  def cache(self):
    """
    Get the DataTables cached data for the selected column.

    Related Pages:

			https://datatables.net/reference/api/column().cache()

    :return:
    """

  def data(self):
    """
    Get the data for the cells in the selected column.

    Related Pages:

			https://datatables.net/reference/api/column().data()

    :return:
    """
    return self.fnc("data()")

  def dataSrc(self):
    """
    Get the data source property for the selected column.

    Related Pages:

			https://datatables.net/reference/api/column().dataSrc()

    :return:
    """

  def footer(self):
    """
    Get the footer node for the selected column.

    Related Pages:

			https://datatables.net/reference/api/column().footer()

    :return:
    """

  def header(self):
    """
    Get the header node for the selected column.

    Related Pages:

			https://datatables.net/reference/api/column().header()

    :return:
    """

  def index(self):
    """
    Get the column index of the selected column.

    Related Pages:

			https://datatables.net/reference/api/column().index()

    :return:
    """
    return JsObjects.JsNumber.JsNumber("%s.index()" % self.getStr())

  def nodes(self):
    """
    Get the cell nodes for the selected column.

    Related Pages:

			https://datatables.net/reference/api/column().nodes()

    :return:
    """
    self._js.append("nodes()")
    return self

  def jquery_nodes(self):
    """
    Get the cell nodes for the selected column.

    Related Pages:

			https://datatables.net/reference/api/column().nodes()

    :return:
    """
    self.nodes()
    self._js.append("to$()")
    return JsQuery.JQuery(jqId=self.toStr())

  def order(self):
    """
    Order the table by the selected column.

    Related Pages:

			https://datatables.net/reference/api/column().order()

    :return:
    """

  def search(self, jsData):
    """
    Search for data in the selected column.

    Related Pages:

			https://datatables.net/reference/api/column().search()

    :return:
    """
    return self.fnc("search(%s)" % JsUtils.jsConvertData(jsData, None))

  def visible(self):
    """
    Get / set the visibility of a single selected column.

    Related Pages:

			https://datatables.net/reference/api/column().visible()

    :return: DataTables API instance for chaining
    """
    return self.fnc("visible()")

  def draw(self, target=None):
    """
    Redraw the DataTables in the current context, optionally updating ordering, searching and paging as required.

    Related Pages:

			https://datatables.net/reference/api/draw()

    :return: DataTables API instance for chaining
    """
    if target is not None:
      return self.fnc("draw(%s)" % JsUtils.jsConvertData(target, None))

    return self.fnc("draw()")


class RowChildAPI(JsPackage):
  lib_selector = 'child'
  lib_set_var = False

  def remove(self):
    pass

  def show(self):
    pass

  def hide(self):
    """
    Hide the child row(s) of a parent row.

    Related Pages:

			https://datatables.net/reference/api/row().child.hide()

    :return:
    """
    return self.fnc("hide()")

  def isShown(self):
    pass


class RowAPI(JsPackage):
  lib_alias = {'js': "datatables", 'css': 'datatables'}
  lib_selector = 'row'
  lib_set_var = False

  def _mapVarId(self, strFnc, varId):
    """
    Change the varIs for row.add.
    This is done at class level and not object level on the Javascript side

    This mapping is done according to the Package API definition

    Related Pages:

			https://datatables.net/reference/api/

    :param strFnc: The function string
    :param varId: The object reference for the Javascript side

    :return: The object reference on the Javascript side
    """
    if strFnc.startswith("add("):
      return varId[:-2]

    return varId

  def deselect(self):
    """
    This method simply deselects a single row that has been found by the row() selector method.

    Related Pages:

			https://datatables.net/reference/api/row().deselect()

    :return: DataTables API instance for chaining
    """
    return self.fnc("deselect()")

  def select(self):
    """
    This method simply selects a single row that has been found by the row() selector method.

    Related Pages:

			https://datatables.net/reference/api/row().select()

    :return: DataTables API instance for chaining
    """
    return self.fnc("select()")

  def scrollTo(self, animate=True):
    """
    Redraw the table's scrolling display to show the row selected by the row() method.

    Related Pages:

			https://datatables.net/reference/api/row().scrollTo()

    :param animate: Animate the scroll (true) or not (false).

    :return: DataTables API instance for chaining
    """
    return self.fnc("scrollTo()")

  def cache(self, dtype):
    """
    Get the DataTables cached data for the selected row.

    Related Pages:

			https://datatables.net/reference/api/row().cache()

    :param dtype: Specify which cache the data should be read from. Can take one of two values: search or order.
                  Defaults to order if no value is given.

    :return: DataTables API instance for chaining
    """
    if dtype not in ("search", "order"):
      raise Exception("dtype %s not recognised" % dtype)

    dtype = JsUtils.jsConvertData(dtype, None)
    return self.fnc("scrollTo(%s)" % dtype)

  def data(self):
    """
    Retrieve the data for the whole table, in row index order.

    Related Pages:

			https://datatables.net/reference/api/row().data()

    :return: DataTables API instance for chaining
    """
    return self.fnc("data()")

  def id(self, hash=True):
    """
    This method can be used to get a row's id, as specified by the row's data and the rowId option.
    Optionally it can also prepend a hash (#) to the row id allowing it to then easily be used as a selector.

    Related Pages:

			https://datatables.net/reference/api/row().id()

    :param hash: Append a hash (#) to the start of the row id. This can be useful for then using the id as a selector

    :return:
    """
    hash = JsUtils.jsConvertData(hash, None)
    return self.fnc("id(%s)" % hash)

  def index(self):
    """
    Get the row index of the selected row.

    Related Pages:

			https://datatables.net/reference/api/row().index()

    :return: Row index
    """
    return JsObjects.JsNumber.JsNumber("%s.index()" % self.getStr())

  def invalidate(self, source=None):
    """
    Invalidate the data held in DataTables for the selected row.

    Related Pages:

			https://datatables.net/reference/api/row().invalidate()

    :param source:

    :return:
    """
    return self.fnc("invalidate(%s)")

  def node(self):
    """
    Get the row TR node for the selected row.

    Related Pages:

			https://datatables.net/reference/api/row().node()

    :return:
    """
    return self.fnc("node()")

  def jquery_node(self):
    """
    Get the cell nodes for the selected column.

    Related Pages:

			https://datatables.net/reference/api/column().nodes()

    :return:
    """
    self.node()
    self._js.append("to$()")
    return JsQuery.JQuery(src=self.src, selector=self.toStr(), setVar=False)

  def remove(self, update=False):
    """
    Delete the selected row from the DataTable.

    Related Pages:

			https://datatables.net/reference/api/row().remove()

    :param update:

    :return:
    """
    if update:
      self.fnc("remove()")
      return self.draw()

    return self.fnc("remove()")

  def add(self, jsData, toArray=False, update=False):
    """
    Add a new row to the table.

    Related Pages:

			https://datatables.net/reference/api/row.add()

    :param jsData: The input data
    :param toArray: Boolean. Convert a python dictionary to a list
    :param update:

    :return:
    """
    if toArray:
      if isinstance(jsData, list):
        rows = []
        for r in jsData:
          rows.append([r.get(h["title"], '') for h in self._parent.vals['columns']])
        jsData = rows
      else:
        jsData = [jsData.get(h["title"], '') for h in self._parent.vals['columns']]
    jsData = JsUtils.jsConvertData(jsData, None)
    self.fnc("add(%s)" % jsData)
    if update:
      self.draw()
      self._js.append([])
      return self

    self._js.append([])
    return self

  def draw(self, target=None):
    """
    Redraw the DataTables in the current context, optionally updating ordering, searching and paging as required.

    Related Pages:

			https://datatables.net/reference/api/draw()

    :return:
    """
    if target is not None:
      return self.fnc("draw(%s)" % JsUtils.jsConvertData(target, None))

    return self.fnc("draw()")

  def child(self, namespace=True):
    """
    Row child method
    Get / set the child rows of the selected main table row

    Related Pages:

			https://datatables.net/reference/api/row().child

    :param namespace: Boolean to set the level of this method, selected rows or namespace
    """
    if namespace:
      return RowChildAPI(self.src, selector="%s.child" % self.getStr(), setVar=False, parent=self._parent)

    return RowChildAPI(self.src, selector="%s.child()" % self.getStr(), setVar=False, parent=self._parent)


class DatatableAPI(JsPackage):
  lib_alias = {'js': "datatables", 'css': 'datatables'}
  lib_selector = 'datatable'

  def body(self):
    """
    Get the tbody node for the table in the API's context.

    Related Pages:

			https://datatables.net/reference/api/table().body()

    :return:
    """
    return

  @property
  def row(self):
    """
    Link to the single row API

    Related Pages:

			https://datatables.net/reference/api/row()
    """
    return RowAPI(self.src, selector="%s.row()" % self.varId, setVar=False, parent=self._parent)

  @property
  def rows(self):
    """
    Link to the rows API

    Related Pages:

			https://datatables.net/reference/api/rows()
    """
    return RowAPI(self.src, selector="%s.rows()" % self.varId, setVar=False, parent=self._parent)

  def adjust(self):
    """
    Adjust the column sizes of a newly shown table:

    Related Pages:

			https://datatables.net/reference/api/columns.adjust()
    """
    return JsObjects.JsVoid("%s.columns.adjust()" % self.varId)

  def container_jquery(self):
    """
    Get the div container node for the table in the API's context.

    Related Pages:

			https://datatables.net/reference/api/table().container()
    """
    return JsQuery.JQuery(self.src, selector="jQuery(%s.table().container())" % self.varId, setVar=False, parent=self._parent)

  def container_dom(self):
    """
    Get the div container node for the table in the API's context.

    Related Pages:

			https://datatables.net/reference/api/table().container()
    """
    return JsNodeDom.JsDoms(None, report=self.src, varName="%s.table().container()" % self.varId, setVar=False)

  def footer(self):
    """
    Get the tfoot node for the table in the API's context

    Related Pages:

			https://datatables.net/reference/api/table().footer()
    """

  def header(self):
    """
    Get the thead node for the table in the API's context

    Related Pages:

			https://datatables.net/reference/api/table().header()

    :return:
    """

  def nodes(self):
    """
    Get the table node for the table in the API's context.

    Related Pages:

			https://datatables.net/reference/api/table().node()
    """
    return self.fnc("nodes()")

  def jquery_node(self):
    """
    Get the cell nodes for the selected column.

    Related Pages:

			https://datatables.net/reference/api/column().nodes()
    """
    return JsQuery.JQuery(selector="%s.nodes().to$()" % self.varId, setVar=False)

  def clear(self, update=False):
    """
    Clear the table of all data:

    Related Pages:

			https://datatables.net/reference/api/clear()

    :param update: Boolean
    :return:
    """
    if update:
      self.fnc("clear()")
      return self.draw()

    return self.fnc("clear()")

  def data(self):
    """
    Retrieve the data for the whole table, in row index order.

    Related Pages:

			https://datatables.net/reference/api/data()

    :return:
    """
    return JsObjects.JsArray.JsArray.get("%s.data()" % self.varId)

  def destroy(self, remove=False, checkUndefined=False):
    """
    Restore the tables in the current context to its original state in the DOM by removing all of DataTables enhancements,
    alterations to the DOM structure of the table and event listeners.

    Related Pages:

			https://datatables.net/reference/api/destroy()

    :param remove: Boolean, Completely remove the table from the DOM (true) or leave it in the DOM in its original plain un-enhanced HTML state (default, false).
    :param checkUndefined: Boolean

    :return:
    """
    return self.fnc_closure("destroy(%s)" % JsUtils.jsConvertData(remove, None), checkUndefined=checkUndefined)

  def draw(self, target=None):
    """
    Redraw the DataTables in the current context, optionally updating ordering, searching and paging as required.

    Related Pages:

			https://datatables.net/reference/api/draw()

    :return:
    """
    if target is not None:
      return self.fnc_closure("draw(%s)" % JsUtils.jsConvertData(target, None))

    return self.fnc_closure("draw()")

  def order(self, data=None):
    """

    Example


    Related Pages:

			https://datatables.net/reference/api/order()

    :return:
    """
    if data is not None:
      data = JsUtils.jsConvertData(data, None)
      return self.fnc("order(%s)" % data)

    return self.fnc("order()")

  def page(self, action):
    """

    Related Pages:

			https://datatables.net/reference/api/page()

    :return:
    """
    if not action in ("first", "next", "previous", "last"):
      raise Exception("Action not defined")

    return self

  def search(self, jsData):
    """

    Related Pages:

			https://datatables.net/reference/api/search()

    :param jsData:
    :return:
    """

    return self

  def settings(self):
    """

    Related Pages:

			https://datatables.net/reference/api/settings()

    :return:
    """
    return self.fnc("settings()")

  def state(self):
    """
    Get the last saved state of the table.

    Related Pages:

			https://datatables.net/reference/api/state()

    :return:
    """
    return JsObjects.JsObject.JsObject.get("%s.state()" % self.varId)

  def cell(self, cellSelector=None, rowColSelector=None):
    """
    Select cells found by both row and column selectors

    Related Pages:

			https://datatables.net/reference/api/cells()
    https://datatables.net/reference/api/cell()

    :return:
    """
    if cellSelector is not None:
      selector = "%s.cell(%s)" % self.toStr()
    elif rowColSelector is not None:
      selector = "%s.cell(%s, %s)" % (self.toStr(), rowColSelector[0], rowColSelector[1])
    else:
      selector = "%s.cell()" % self.toStr()
    return CellAPI(selector)

  def column(self, colSelector):
    """

    :param colSelector:
    :return:
    """
    if colSelector is not None:
      selector = "%s.column(%s)" % (self.toStr(), JsUtils.jsConvertData(colSelector, None))
    else:
      selector = "%s.column()" % self.toStr()
    return ColumnAPI(selector)

  def columns(self, colSelector):
    """

    :param colSelector:
    :return:
    """
    if colSelector is not None:
      selector = "%s.column(%s)" % (self.varId, JsUtils.jsConvertData(colSelector, None))
    else:
      selector = "%s.column()" % self.varId
    return ColumnAPI(selector)

  def select(self):
    """
    Initialise Select from outside of the constructor

    Related Pages:

			https://datatables.net/reference/api/select()

    TODO add the select true
    :return:
    """
    return SelectAPI(selector="%s.select()", setVar=False)




import os

from epyk.core.html import graph


class ChartJs(object):

  def __init__(self, context):
    self.parent = context
    self.chartFamily = "ChartJs"

  def line(self, record, y_columns=None, x_axis=None, profile=None, width=(100, "%"), height=(330, "px"), options=None, htmlCode=None):
    """
    Description:
    ------------
    Display a line chart from ChartJs

    Related Pages:

      https://www.chartjs.org/samples/latest/scales/logarithmic/line.html

    Attributes:
    ----------
    :param record: List of dict. The Python recordset
    :param y_columns: List. The columns corresponding to keys in the dictionaries in the record
    :param x_axis: String. The column corresponding to a key in the dictionaries in the record
    :param profile:
    :param width: Tuple. The width of the component in the page, default (100, '%')
    :param height: Tuple. The height of the component in the page, default (330, "px")
    :param options:
    :param htmlCode:
    """
    options = options or {}
    options.update({'y_columns': y_columns, 'x_column': x_axis, 'colors': self.parent.context.rptObj.theme.charts, 'attrs': {'fill': None}})
    data = self.parent.context.rptObj.data.chartJs.y(record, y_columns, x_axis)
    line_chart = graph.GraphChartJs.ChartLine(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    line_chart.labels(data['labels'])
    for i, d in enumerate(data['datasets']):
      line_chart.add_dataset(d, data['series'][i])
    return line_chart

  def timeseries(self, record, y_columns=None, x_axis=None, profile=None, options=None, width=(100, "%"), height=(330, "px"), htmlCode=None):
    """
    Description:
    ------------
    Display a line chart from ChartJs

    Related Pages:

      https://www.chartjs.org/samples/latest/scales/logarithmic/line.html

    Attributes:
    ----------
    :param record: List of dict. The Python recordset
    :param y_columns: List. The columns corresponding to keys in the dictionaries in the record
    :param x_axis: String. The column corresponding to a key in the dictionaries in the record
    :param profile:
    :param width: Tuple. The width of the component in the page, default (100, '%')
    :param height: Tuple. The height of the component in the page, default (330, "px")
    :param options:
    :param htmlCode:
    """
    line = self.line(record, y_columns, x_axis, profile, width, height, options, htmlCode)
    return line

  def pie(self, record=None, y_columns=None, x_axis=None, profile=None, width=(100, "%"), height=(330, "px"), options=None, htmlCode=None):
    """
    Description:
    ------------
    Display a pie chart from ChartJs

    Related Pages:

      https://www.chartjs.org/samples/latest/charts/pie.html

    Attributes:
    ----------
    :param record: List of dict. The Python recordset
    :param y_columns: List. The columns corresponding to keys in the dictionaries in the record
    :param x_axis: String. The column corresponding to a key in the dictionaries in the record
    :param profile:
    :param width: Tuple. The width of the component in the page, default (100, '%')
    :param height: Tuple. The height of the component in the page, default (330, "px")
    :param options:
    :param htmlCode:
    """
    options = options or {}
    options.update({'y_columns': y_columns, 'x_column': x_axis, 'colors': self.parent.context.rptObj.theme.charts, 'attrs': {}})
    data = self.parent.context.rptObj.data.chartJs.y(record, y_columns, x_axis)
    line_chart = graph.GraphChartJs.ChartPie(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    line_chart.labels(data['labels'])
    for i, d in enumerate(data['datasets']):
      line_chart.add_dataset(d, data['series'][i])
    return line_chart

  def donut(self, record=None, y_columns=None, x_axis=None, profile=None, width=(100, "%"), height=(330, "px"), options=None, htmlCode=None):
    """
    Description:
    ------------
    Display a donut chart from ChartJs

    Related Pages:

      https://www.chartjs.org/samples/latest/charts/doughnut.html

    Attributes:
    ----------
    :param record: List of dict. The Python recordset
    :param y_columns: List. The columns corresponding to keys in the dictionaries in the record
    :param x_axis: String. The column corresponding to a key in the dictionaries in the record
    :param profile:
    :param width: Tuple. The width of the component in the page, default (100, '%')
    :param height: Tuple. The height of the component in the page, default (330, "px")
    :param options:
    :param htmlCode:
    """
    data = self.parent.context.rptObj.data.chartJs.y(record, y_columns, x_axis)
    dflt_options = {'cutoutPercentage': 50, 'y_columns': y_columns, 'x_column': x_axis, 'colors': self.parent.context.rptObj.theme.charts, 'attrs': {}}
    if options is not None:
      dflt_options.update()
    pie_chart = graph.GraphChartJs.ChartPie(self.parent.context.rptObj, width, height, htmlCode, dflt_options, profile)
    pie_chart.labels(data['labels'])
    for i, d in enumerate(data['datasets']):
      pie_chart.add_dataset(d, data['series'][i])
    return pie_chart

  def area(self, record, y_columns=None, x_axis=None, profile=None, width=(100, "%"), height=(330, "px"), options=None, htmlCode=None):
    """
    Description:
    ------------
    Display a area chart from ChartJs

    Related Pages:

      https://www.chartjs.org/samples/latest/charts/area/line-stacked.html

    Attributes:
    ----------
    :param record: List of dict. The Python recordset
    :param y_columns: List. The columns corresponding to keys in the dictionaries in the record
    :param x_axis: String. The column corresponding to a key in the dictionaries in the record
    :param profile:
    :param width: Tuple. The width of the component in the page, default (100, '%')
    :param height: Tuple. The height of the component in the page, default (330, "px")
    :param options:
    :param htmlCode:
    """
    options = options or {}
    options.update({'y_columns': y_columns, 'x_column': x_axis, 'colors': self.parent.context.rptObj.theme.charts, 'attrs': {"opacity": 0.2}})
    data = self.parent.context.rptObj.data.chartJs.y(record, y_columns, x_axis)
    line_chart = graph.GraphChartJs.ChartLine(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    line_chart.labels(data['labels'])
    for i, d in enumerate(data['datasets']):
      line_chart.add_dataset(d, data['series'][i], opacity=0.2)
      line_chart.dataset().fill = True
    return line_chart

  def step(self, record, y_columns=None, x_axis=None, profile=None, width=(100, "%"), height=(330, "px"), options=None, htmlCode=None):
    """
    Description:
    ------------
    Display a step chart from ChartJs

    Related Pages:

      https://www.chartjs.org/samples/latest/scales/linear/step-size.html

    Attributes:
    ----------
    :param record: List of dict. The Python recordset
    :param y_columns: List. The columns corresponding to keys in the dictionaries in the record
    :param x_axis: String. The column corresponding to a key in the dictionaries in the record
    :param profile:
    :param width: Tuple. The width of the component in the page, default (100, '%')
    :param height: Tuple. The height of the component in the page, default (330, "px")
    :param options:
    :param htmlCode:
    """
    options = options or {}
    options.update({'y_columns': y_columns, 'x_column': x_axis, 'colors': self.parent.context.rptObj.theme.charts, 'attrs': {"fill": None}})
    data = self.parent.context.rptObj.data.chartJs.y(record, y_columns, x_axis)
    line_chart = graph.GraphChartJs.ChartLine(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    line_chart.labels(data['labels'])
    for i, d in enumerate(data['datasets']):
      line_chart.add_dataset(d, data['series'][i])
      line_chart.dataset().steppedLine = 'before'
    return line_chart

  def bar(self, record, y_columns=None, x_axis=None, profile=None, width=(100, "%"), height=(330, "px"), options=None, htmlCode=None):
    """
    Description:
    ------------
    Display a bar chart from ChartJs

    Related Pages:

      https://www.chartjs.org/samples/latest/scriptable/bar.html

    Attributes:
    ----------
    :param record: List of dict. The Python recordset
    :param y_columns: List. The columns corresponding to keys in the dictionaries in the record
    :param x_axis: String. The column corresponding to a key in the dictionaries in the record
    :param profile:
    :param width: Tuple. The width of the component in the page, default (100, '%')
    :param height: Tuple. The height of the component in the page, default (330, "px")
    :param options:
    :param htmlCode:
    """
    options = options or {}
    options.update({'y_columns': y_columns, 'x_column': x_axis, 'colors': self.parent.context.rptObj.theme.charts, 'attrs': {}})
    data = self.parent.context.rptObj.data.chartJs.y(record, y_columns, x_axis)
    bar_chart = graph.GraphChartJs.ChartBar(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    bar_chart.labels(data['labels'])
    for i, d in enumerate(data['datasets']):
      bar_chart.add_dataset(d, data['series'][i])
    bar_chart.options.scales.y_axis().ticks.beginAtZero = True
    return bar_chart

  def custom(self, record, y_columns=None, x_axis=None, profile=None, width=(100, "%"), height=(330, "px"), options=None, htmlCode=None):
    """
    Description:
    ------------
    Display a bar chart from ChartJs

    Related Pages:

      https://www.chartjs.org/samples/latest/scriptable/bar.html

    Attributes:
    ----------
    :param record: List of dict. The Python recordset
    :param y_columns: List. The columns corresponding to keys in the dictionaries in the record
    :param x_axis: String. The column corresponding to a key in the dictionaries in the record
    :param profile:
    :param width: Tuple. The width of the component in the page, default (100, '%')
    :param height: Tuple. The height of the component in the page, default (330, "px")
    :param options:
    :param htmlCode:
    """
    if self.parent.context.rptObj.ext_packages is None:
      self.parent.context.rptObj.ext_packages = {}
    chartjs_defined_package = {
      'funnel-chart-js': {
        'version': '1.1.2',
        'req': [{'alias': 'Chart.js'}],
        'website': 'https://www.npmjs.com/package/chartjs-plugin-funnel',
        'modules': [
          {'script': 'chart.funnel.min.js', 'path': '', 'cdnjs': '%s/funnel-chart-js/dist/' % options['npm_path'].replace("\\", "/")},
        ]},
      'chartjs-chart-sankey': {
        'version': '0.1.3',
        'website': 'https://www.npmjs.com/package/chartjs-chart-sankey',
        'req': [{'alias': 'Chart.js'}],
        'modules': [
          {'script': 'chartjs-chart-sankey.min.js', 'path': '', 'cdnjs': '%s/chartjs-chart-sankey/dist/' % options['npm_path'].replace("\\", "/")}
        ]}
    }
    if options.get('npm') is not None and options['npm'] not in chartjs_defined_package:
      folder = "dist"
      if not os.path.exists('%s/%s/%s' % (options['npm_path'].replace("\\", "/"), options['npm'], folder)):
        folder = 'build'
        if not os.path.exists('%s/%s/%s' % (options['npm_path'].replace("\\", "/"), options['npm'], folder)):
          raise Exception("Missing module %s/%s/dist or build in the package - use web npm feature to install package to your app " % (options['npm_path'].replace("\\", "/"), options['npm']))

      chartjs_defined_package[options['npm']] = {
        'req': [{'alias': 'Chart.js'}], 'version': 'N/A',
        'modules': [
          {'script': '%s.min.js' % options.get('script', options['npm']), 'path': '', 'cdnjs': '%s/%s/%s' % (options['npm_path'].replace("\\", "/"), options['npm'], folder) },
      ]}

      del options['npm_path']
      if 'script' in options:
        del options['script']

    self.parent.context.rptObj.ext_packages.update(chartjs_defined_package)
    options = options or {}
    options.update({'y_columns': y_columns, 'x_column': x_axis, 'colors': self.parent.context.rptObj.theme.charts, 'attrs': {}})
    data = self.parent.context.rptObj.data.chartJs.y(record, y_columns, x_axis)
    bar_chart = graph.GraphChartJs.ChartExts(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    bar_chart.labels(data['labels'])
    for i, d in enumerate(data['datasets']):
      bar_chart.add_dataset(d, data['series'][i])
    bar_chart.options.scales.y_axis().ticks.beginAtZero = True
    return bar_chart

  def hbar(self, record, y_columns=None, x_axis=None, profile=None, width=(100, "%"), height=(330, "px"), options=None, htmlCode=None):
    """
    Description:
    ------------
    Display a horizontal bar chart from ChartJs

    Related Pages:

      https://www.chartjs.org/samples/latest/scriptable/bar.html

    Attributes:
    ----------
    :param record: List of dict. The Python recordset
    :param y_columns: List. The columns corresponding to keys in the dictionaries in the record
    :param x_axis: String. The column corresponding to a key in the dictionaries in the record
    :param profile:
    :param width: Tuple. The width of the component in the page, default (100, '%')
    :param height: Tuple. The height of the component in the page, default (330, "px")
    :param options:
    :param htmlCode:
    """
    options = options or {}
    options.update({'y_columns': y_columns, 'x_column': x_axis, 'colors': self.parent.context.rptObj.theme.charts, 'attrs': {}})
    data = self.parent.context.rptObj.data.chartJs.y(record, y_columns, x_axis)
    bar_chart = graph.GraphChartJs.ChartHBar(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    bar_chart.labels(data['labels'])
    for i, d in enumerate(data['datasets']):
      bar_chart.add_dataset(d, data['series'][i])
    return bar_chart

  def multi(self, type, record=None, y_columns=None, x_axis=None, profile=None, options=None, width=(100, "%"), height=(330, "px"), htmlCode=None):
    """
    Description:
    ------------
    Display a multi chart from ChartJs

    Related Pages:

      https://www.chartjs.org/samples/latest/charts/combo-bar-line.html

    Attributes:
    ----------
    :param record: List of dict. The Python recordset
    :param y_columns: List. The columns corresponding to keys in the dictionaries in the record
    :param x_axis: String. The column corresponding to a key in the dictionaries in the record
    :param profile:
    :param width: Tuple. The width of the component in the page, default (100, '%')
    :param height: Tuple. The height of the component in the page, default (330, "px")
    :param options:
    :param htmlCode:
    """
    options = options or {}
    options.update({'y_columns': y_columns, 'x_column': x_axis, 'colors': self.parent.context.rptObj.theme.charts, 'attrs': {}})
    data = self.parent.context.rptObj.data.chartJs.y(record, y_columns, x_axis)
    bar_chart = graph.GraphChartJs.ChartBar(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    bar_chart.labels(data['labels'])
    for i, d in enumerate(data['datasets']):
      bar_chart.add_dataset(d, data['series'][i])
    bar_chart._attrs['type'] = type
    bar_chart.options.scales.y_axis().ticks.beginAtZero = True
    return bar_chart

  def scatter(self, record, y_columns=None, x_axis=None, profile=None, width=(100, "%"), height=(330, "px"), options=None, htmlCode=None):
    """
    Description:
    ------------
    Display a scatter chart from ChartJs

    Related Pages:

      https://www.chartjs.org/samples/latest/charts/scatter/basic.html

    Attributes:
    ----------
    :param record: List of dict. The Python recordset
    :param y_columns: List. The columns corresponding to keys in the dictionaries in the record
    :param x_axis: String. The column corresponding to a key in the dictionaries in the record
    :param profile:
    :param width: Tuple. The width of the component in the page, default (100, '%')
    :param height: Tuple. The height of the component in the page, default (330, "px")
    :param options:
    :param htmlCode:
    """
    options = options or {}
    options.update({'y_columns': y_columns, 'x_column': x_axis, 'colors': self.parent.context.rptObj.theme.charts, 'rDim': None, 'attrs': {}})
    data = self.parent.context.rptObj.data.chartJs.xyz(record, y_columns, x_axis, None)
    line_chart = graph.GraphChartJs.ChartScatter(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    line_chart.labels(data['labels'])
    for i, d in enumerate(data['datasets']):
      line_chart.add_dataset(d, data['series'][i])
    return line_chart

  def bubble(self, record, y_columns=None, x_axis=None, r_values=None, profile=None, width=(100, "%"), height=(330, "px"), options=None, htmlCode=None):
    """
    Description:
    ------------
    Display a bubble chart from ChartJs

    Related Pages:

      https://www.chartjs.org/samples/latest/scriptable/bubble.html

    Attributes:
    ----------
    :param record: List of dict. The Python recordset
    :param y_columns: List. The columns corresponding to keys in the dictionaries in the record
    :param x_axis: String. The column corresponding to a key in the dictionaries in the record
    :param profile:
    :param width: Tuple. The width of the component in the page, default (100, '%')
    :param height: Tuple. The height of the component in the page, default (330, "px")
    :param options:
    :param htmlCode:
    """
    options = options or {}
    options.update({'y_columns': y_columns, 'x_column': x_axis, 'z_columns': r_values, 'colors': self.parent.context.rptObj.theme.charts, 'rDim': None, 'attrs': {}})
    data = self.parent.context.rptObj.data.chartJs.xyz(record, y_columns, x_axis, r_values)
    line_chart = graph.GraphChartJs.ChartBubble(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    line_chart.labels(data['labels'])
    for i, d in enumerate(data['datasets']):
      line_chart.add_dataset(d, data['series'][i])
    return line_chart

  def polar(self, record, y_columns=None, x_axis=None, profile=None, width=(100, "%"), height=(330, "px"), options=None, htmlCode=None):
    """
    Description:
    ------------
    Display a bubble chart from ChartJs

    Related Pages:

      https://www.chartjs.org/samples/latest/charts/polar-area.html

    Attributes:
    ----------
    :param record: List of dict. The Python recordset
    :param y_columns: List. The columns corresponding to keys in the dictionaries in the record
    :param x_axis: String. The column corresponding to a key in the dictionaries in the record
    :param profile:
    :param width: Tuple. The width of the component in the page, default (100, '%')
    :param height: Tuple. The height of the component in the page, default (330, "px")
    :param options:
    :param htmlCode:
    """
    options = options or {}
    options.update({'y_columns': y_columns, 'x_column': x_axis, 'colors': self.parent.context.rptObj.theme.charts, 'attrs': {}})
    data = self.parent.context.rptObj.data.chartJs.y(record, y_columns, x_axis)
    polar_chart = graph.GraphChartJs.ChartPolar(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    polar_chart.labels(data['labels'])
    for i, d in enumerate(data['datasets']):
      polar_chart.add_dataset(d, data['series'][i])
    return polar_chart

  def radar(self, record, y_columns=None, x_axis=None, profile=None, width=(100, "%"), height=(330, "px"), options=None, htmlCode=None):
    """
    Description:
    ------------
    Display a bubble chart from ChartJs

    Related Pages:

      https://www.chartjs.org/samples/latest/charts/radar.html

    Attributes:
    ----------
    :param record: List of dict. The Python recordset
    :param y_columns: List. The columns corresponding to keys in the dictionaries in the record
    :param x_axis: String. The column corresponding to a key in the dictionaries in the record
    :param profile:
    :param width: Tuple. The width of the component in the page, default (100, '%')
    :param height: Tuple. The height of the component in the page, default (330, "px")
    :param options:
    :param htmlCode:
    """
    options = options or {}
    options.update({'y_columns': y_columns, 'x_column': x_axis, 'colors': self.parent.context.rptObj.theme.charts, 'attrs': {}})
    data = self.parent.context.rptObj.data.chartJs.y(record, y_columns, x_axis)
    radar_chart = graph.GraphChartJs.ChartRadar(self.parent.context.rptObj, width, height, htmlCode, options, profile)
    radar_chart.labels(data['labels'])
    for i, d in enumerate(data['datasets']):
      radar_chart.add_dataset(d, data['series'][i])
    return radar_chart




from epyk.core.css.styles.classes import CssStyle

from epyk.core.css import Defaults as Defaults_css


class CssTextBold(CssStyle.Style):
  _attrs = {'font-weight': 'bold'}


class CssFormula(CssStyle.Style):
  """
  CSS Style for the formulas component
  """
  _attrs = {'padding': 0, 'margin': 0}
  _focus = {'outline': 0, 'border': 'none', 'box-shadow': 'none'}
  classname = "math"


class CssTitle1(CssStyle.Style):
  _attrs = {'padding': '0 0 5px 0', 'font-weight': 'bold', 'text-transform': 'uppercase',
            'white-space': 'pre-wrap', 'border-bottom': '1px dashed black', 'border-width': '2px',
            'margin-bottom': '5px'}

  def customize(self):
    self.css({"color": self.rptObj.theme.greys[7], "border-color": self.rptObj.theme.greys[9],
              'font-size': Defaults_css.font(10)})


class CssTitle2(CssStyle.Style):
  _attrs = {'padding': 0, 'margin-top': '5px', 'font-weight': 'bold', 'text-transform': 'uppercase',
            'white-space': 'pre-wrap'}

  def customize(self):
    self.css({"color": self.rptObj.theme.colors[7], 'font-size': Defaults_css.font(8)})


class CssTitle3(CssStyle.Style):
  _attrs = {'padding': 0, 'margin-top': '5px', 'font-weight': 'bold', 'text-transform': 'uppercase',
            'white-space': 'pre-wrap'}

  def customize(self):
    self.css({"color": self.rptObj.theme.colors[7], 'font-family': Defaults_css.Font.family,
              'font-size': Defaults_css.font(2)})


class CssTitle4(CssStyle.Style):
  _attrs = {'padding': 0, 'margin': '5px 0 0 0', 'font-weight': 'bold', 'width': '100%', 'white-space': 'pre-wrap'}

  def customize(self):
    self.css({"color": self.rptObj.theme.greys[5], 'font-size': Defaults_css.font(1)})


class CssTitle(CssStyle.Style):
  _attrs = {'padding': 0, 'margin-bottom': 0, 'white-space': 'pre-wrap', 'font-weight': 'bold'}

  def customize(self):
    self.css({'font-size': Defaults_css.font(2)})


class CssNumberCenter(CssStyle.Style):
  _attrs = {'width': '100%', 'text-align': 'center', 'padding': 0, 'margin-bottom': 0, 'white-space': 'pre-wrap',
            'font-weight': 'bold'}

  def customize(self):
    self.css({'font-size': '%s%s' % (Defaults_css.Font.header_size, Defaults_css.Font.unit),
              'font-family': Defaults_css.Font.family})


class CssMarkRed(CssStyle.Style):
  _attrs = {'background': 'none'}

  def customize(self):
    self.css({"color": self.rptObj.theme.danger[1],
              'font-size': '%s%s' % (Defaults_css.Font.size, Defaults_css.Font.unit)})


class CssMarkBlue(CssStyle.Style):
  _attrs = {'background': 'none', 'font-weight': 'bold'}

  def customize(self):
    self.css({"color": self.rptObj.theme.colors[7],
              'font-size': '%s%s' % (Defaults_css.Font.size, Defaults_css.Font.unit)})


class CssTextWithBorder(CssStyle.Style):
  _attrs = {'border': '1px solid', 'padding': '5px', 'margin': '10px'}
  _selectors = {'child': 'fieldset'}

  def customize(self):
    self.css({"background-color": self.rptObj.theme.greys[0]})


class CssCheckMark(CssStyle.Style):
  _attrs = {'text-align': 'center', 'display': 'inline-block',
            'font-family': 'FontAwesome', 'height': '18px', 'width': '18px'}

  def customize(self):
    self.css({"background-color": self.rptObj.theme.greys[0], "color": self.rptObj.theme.greys[9]})
    self.hover.css({'color': 'white', 'background-color': self.rptObj.theme.colors[9]})


class CssTextItem(CssStyle.Style):
  _attrs = {'cursor': 'pointer', 'width': '200px', 'padding': '5px 5px 5px 20px'}

  def customize(self):
    self.hover.css({"color": self.rptObj.theme.greys[-1], "background": self.rptObj.theme.colors[2]})


class CssTextNotSelectable(CssStyle.Style):
  _attrs = {'-moz-user-select': 'none', "user-select": 'none', '-khtml-user-select': 'none',
            '-webkit-user-select': 'none', '-ms-user-select': 'none', "-o-user-select": 'none'}




from epyk.core.html.options import Options


class OptionMaps(Options):

  @property
  def zoom(self):
    """
    Description:
    ------------
    The starting value of the editor. Can be a string, or a document object.

    Related Pages:

      https://codemirror.net/doc/manual.html#config
    """
    return self._config_get(10)

  @zoom.setter
  def zoom(self, num):
    self._config(num)

  @property
  def center(self):
    """
    Description:
    ------------
    The starting value of the editor. Can be a string, or a document object.

    Related Pages:

      https://codemirror.net/doc/manual.html#config
    """
    return self._config_get(None)

  @center.setter
  def center(self, point):
    self.js_type['center'] = True
    if isinstance(point, tuple):
      self._config("new google.maps.LatLng(%s, %s)" % (point[0], point[1]))
    else:
      self._config("new google.maps.LatLng(%s)" % point)

  @property
  def mapTypeId(self):
    """
    Description:
    ------------
    The starting value of the editor. Can be a string, or a document object.

    Related Pages:

      https://codemirror.net/doc/manual.html#config
    """
    return self._config_get("google.maps.MapTypeId.ROADMAP")

  @mapTypeId.setter
  def mapTypeId(self, type_map):
    self.js_type['mapTypeId'] = True
    self._config("google.maps.MapTypeId.%s" % type_map.upper())




from epyk.core.html import graph


class Canvas(object):

  def __init__(self, context):
    self.parent = context

  def new(self, height=(400, "px"), width=(100, "%")):
    """
    Description:
    ------------
    The HTML <canvas> tag is used to draw graphics, on the fly, via scripting (usually JavaScript).

    However, the <canvas> element has no drawing abilities of its own (it is only a container for graphics) - you must use a script to actually draw the graphics.

    The getContext() method returns an object that provides methods and properties for drawing on the canvas.

    This reference will cover the properties and methods of the getContext("2d") object, which can be used to draw text, lines, boxes, circles, and more - on the canvas

    Related Pages:

			https://www.w3schools.com/tags/ref_canvas.asp

    Attributes:
    ----------
    :param height:
    :param width:
    """
    if not isinstance(width, tuple):
      width = (width, "px")
    html_svg = graph.GraphCanvas.Canvas(self.parent.context.rptObj, width, height)
    return html_svg



import os
from epyk.core import html


class Network(object):
  """

  """

  def __init__(self, context):
    self.context = context

  def comments(self, htmlCode, recordSet=None, width=(100, '%'), height=(200, 'px'), profile=None, options=None):
    """
    Description:
    ------------
    Python wrapper to a div item composed to several sub html items to display message

    Usage::

      db = report.db(database="test.db")
      report.comments('Test', dbService={'db': db, 'com_table': 'comments', 'reply_table': 'replyComments', 'reply_service': 'post_reply/url', 'user_coms': 'user_comments', 'privacy': 'public', 'service': your/url})

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlMessaging.Comments`

    Related Pages:

        https://leaverou.github.io/bubbly/
    http://manos.malihu.gr/jquery-custom-content-scroller/

    Attributes:
    ----------
    :param htmlCode:
    :param width:
    :param height:
    :param profile:
    """
    dflt_options = {'readonly': True, 'markdown': True, 'dated': True}
    if options is not None:
      dflt_options.update(options)
    return html.HtmlNetwork.Comments(self.context.rptObj, recordSet, width, height, htmlCode, dflt_options, profile)

  def chat(self, htmlCode, recordSet=None, width=(100, '%'), height=(200, 'px'), profile=None, options=None):
    """

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlMessaging.Chat`

    Attributes:
    ----------
    :param htmlCode:
    :param width:
    :param height:
    :param profile:
    """
    dflt_options = {'readonly': False, 'markdown': True, 'dated': True}
    if options is not None:
      dflt_options.update(options)
    return html.HtmlNetwork.Chat(self.context.rptObj, recordSet, width, height, htmlCode, dflt_options, profile)

  def bot(self, htmlCode, width=(100, '%'), height=(200, 'px'), profile=None, options=None):
    """

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlMessaging.Bot`

    Attributes:
    ----------
    :param htmlCode:
    :param width:
    :param height:
    :param profile:
    """
    dflt_options = {'readonly': False, 'markdown': True, 'dated': True}
    if options is not None:
      dflt_options.update(options)
    return html.HtmlNetwork.Bot(self.context.rptObj, width, height, htmlCode, dflt_options, profile)

  def alert(self, type, value="", width=(320, 'px'), height=(None, None), htmlCode=None, options=None, profile=False):
    """
    Description:
    ------------
    Function to add when the python run some tags to put on the top of your report messages.

    The type of the messages can be different according to its criticallity.
    This is fully defined and #driven in the Python and visible in the browser when the page is ready

    All the notification can be hidden directly from the report by setting the flag alerts = False
    e.g: rptObj.alerts = False

    Usage::

      report.ui.messaging.alert('WARNING', 'Server URL not recognized', 'Please check')

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlMessaging.Alert`

    Related Pages:

			https://getbootstrap.com/docs/4.0/components/alerts/

    Attributes:
    ----------
    :param value: The content of the notification
    :param width:
    :param height:
    :param htmlCode:
    :param options:
    :param profile:
    """
    dflt_options = {"time": 1000, 'delay': 1000, 'type': type, 'close': True, 'markdown': True,
                    'classes': ['alert', 'alert-%s' % type]}
    if options is not None:
      dflt_options.update(options)
    return html.HtmlNetwork.Alert(self.context.rptObj, value, width, height, htmlCode, dflt_options, profile)

  def danger(self, value="", htmlCode=None, width=(320, 'px'), height=(None, None), options=None, profile=False):
    """

    Description:
    ------------Function to add when the python run some tags to put on the top of your report messages.

    The type of the messages can be different according to its criticality.
    This is fully defined and #driven in the Python and visible in the browser when the page is ready

    All the notification can be hidden directly from the report by setting the flag alerts = False
    e.g: rptObj.alerts = False

    Usage::

      rptObj.ui.messaging.alert('danger', 'Server URL not recognized', 'Please check')

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlMessaging.Alert`

    Related Pages:

        https://getbootstrap.com/docs/4.0/components/alerts/

    Attributes:
    ----------
    :param value: The content of the notification
    :param width:
    :param height:
    :param htmlCode:
    :param options:
    :param profile:
    """
    return self.alert('danger', value, width, height, htmlCode, options, profile)

  def info(self, value="", htmlCode=None, width=(320, 'px'), height=(None, None), options=None, profile=False):
    """
    Description:
    ------------
    Function to add when the python run some tags to put on the top of your report messages.
    The type of the messages can be different according to its criticallity.
    This is fully defined and #driven in the Python and visible in the browser when the page is ready

    Usage::

      rptObj.ui.messaging.alert('info', 'Server URL not recognized', 'Please check')

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlMessaging.Alert`

    Related Pages:

        https://getbootstrap.com/docs/4.0/components/alerts/

    Attributes:
    ----------
    :param value: The content of the notification
    :param width:
    :param height:
    :param htmlCode:
    :param options:
    :param profile:
    """
    return self.alert('info', value, width, height, htmlCode, options, profile)

  def success(self, value="", htmlCode=None, width=(320, 'px'), height=(None, None), options=None, profile=False):
    """
    Description:
    ------------
    unction to add when the python run some tags to put on the top of your report messages.
    The type of the messages can be different according to its criticallity.
    This is fully defined and #driven in the Python and visible in the browser when the page is ready

    Usage::

      rptObj.ui.messaging.alert('success', 'Server URL not recognized', 'Please check')

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlMessaging.Alert`

    Related Pages:

        https://getbootstrap.com/docs/4.0/components/alerts/

    Attributes:
    ----------
    :param value: The content of the notification
    :param width:
    :param height:
    :param htmlCode:
    :param options:
    :param profile:
    """
    return self.alert('success', value, width, height, htmlCode, options, profile)

  def warning(self, value="", htmlCode=None, width=(320, 'px'), height=(None, None), options=None, profile=False):
    """
    Description:
    ------------
    Function to add when the python run some tags to put on the top of your report messages.
    The type of the messages can be different according to its criticallity.
    This is fully defined and #driven in the Python and visible in the browser when the page is ready

    Usage::

      rptObj.ui.messaging.alert('warning', 'Server URL not recognized', 'Please check')

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlMessaging.Alert`

    Related Pages:

        https://getbootstrap.com/docs/4.0/components/alerts/

    Attributes:
    ----------
    :param value: The content of the notification
    :param width:
    :param height:
    :param htmlCode:
    :param options:
    :param profile:
    """
    return self.alert('warning', value, width, height, htmlCode, options, profile)

  def news(self, value="", htmlCode=None, width=(320, 'px'), height=(None, None), options=None, profile=False):
    """
    Description:
    ------------

    Usage::

      b = rptObj.ui.button("Display")
      n = rptObj.ui.messaging.news("This is a title", "This is the content", link_script="TestSlider")
      b.click(n.jsGenerate("Updated content", isPyData=True))

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlMessaging.News`

    Attributes:
    ----------
    :param value:
    :param htmlCode:
    :param width:
    :param height:
    :param options:
    :param profile:
    """
    dflt_options = {"dated": True, 'markdown': True}
    if options is not None:
      dflt_options.update(options)
    return html.HtmlNetwork.News(self.context.rptObj, value, width, height, htmlCode, dflt_options, profile)

  def room(self, img, htmlCode=None, width=(60, 'px'), height=(60, 'px'), options=None, profile=False):
    """
    Description:
    ------------

    Attributes:
    ----------
    :param img: String. The image path on the server or locally to be used
    :param htmlCode: String. Optional. The id for this component
    :param width:
    :param height:
    :param options:
    :param profile:
    """
    dflt_options = {"dated": True, 'markdown': True}
    if options is not None:
      dflt_options.update(options)
    return html.HtmlNetwork.Room(self.context.rptObj, img, width, height, htmlCode, dflt_options, profile)

  def dropfile(self, placeholder='Drop your files here', tooltip=None, options=None, profile=None):
    """
    Description:
    ------------
    Add an HTML component to drop files. The files will be dropped by default to the OUTPUT folder of the defined environment.
    Files will also be recorded in the database in order to ensure that those data will not be shared.
    The data sharing is and should be defined only by the user from the UI.

    Underlying HTML Objects:

      - :class:`epyk.core.html.HtmlFiles.DropFile`

    Usage::

      rptObj.ui.network.dropfile()

    Related Pages:
Attributes:
    ----------
    :param placeholder:
    :param tooltip:
    :param profile:

    :rtype: html.HtmlNetwork.DropFile

    :return:
    """
    return html.HtmlNetwork.DropFile(self.context.rptObj, placeholder, tooltip, options, profile)

  def upload(self, icon=None, width=(25, 'px'), height=(25, 'px'), htmlCode=None, options=None, profile=None):
    if icon is None:
      icon = "fas fa-file-upload"
    file = self.context.rptObj.ui.icons.awesome(icon, width=width, height=height, htmlCode=htmlCode, profile=profile)
    return file

  def download(self, name, icon=None, path=None, width=(25, 'px'), height=(25, 'px'), htmlCode=None, options=None, profile=None):
    """
    Description:
    ------------

    :param name:
    :param icon:
    :param path:
    :param width:
    :param height:
    :param htmlCode:
    :param options:
    :param profile:
    """
    mapped_file = {"excel": 'far fa-file-excel', 'pdf': 'far fa-file-pdf', 'code': 'far fa-file-code', 'csv': 'fas fa-file-csv',
                   'word': 'fa-file-word'}
    extension = name.split(".")[-1]
    if extension in mapped_file:
      icon = mapped_file[extension]
    if icon is None:
      icon = "fas fa-file-upload"

    file = self.context.rptObj.ui.icons.awesome(icon, width=width, height=height, htmlCode=htmlCode, profile=profile)
    file.tooltip(r"Download file: %(path)s\%(name)s" % {"path": path, "name": name})
    if path is not None:
      file.click(['''
        var link = document.createElement("a");
        link.href = 'file://localhost/%(path)s';
        link.target="_blank";
        link.setAttribute('download', '%(name)s');
        document.body.appendChild(link);
        link.click();
        document.body.removeChild(link);
        ''' % {"path": os.path.join(path, name), "name": name}])
    return file



"""
Theme module for the Dark classes

All the themes are following the same structure and the are all inheriting from Theme
this will allow them to be converted to HTML to be able to check the colors.

A theme class must have the below class variables defined

  name    : The name of the theme to displayed in the navbar
  charts  : A list with the colors used for the series in the charts
  colors  : A list with the colors used for the HTML components
  greys   : A list with the shade of greys to be used within this theme
  warning : A tuple with the two colours used for warning (light, dark)
  danger  : A tuple with the two colours used for danger (light, dark)
  success : A tuple with the two colours used for success (light, dark)
"""

from epyk.core.css.themes import Theme


class Dark(Theme.Theme):
  _charts = [
    '#009999', '#336699', '#ffdcb9',
    '#cc99ff', '#b3d9ff', '#ffff99',
    '#000066', '#b2dfdb', '#80cbc4',
    '#e8f5e9', '#c8e6c9', '#a5d6a7', # green
    '#ffebee', '#ffcdd2', '#ef9a9a', # red
    '#f3e5f5', '#e1bee7', '#ce93d8', # purple
    '#ede7f6', '#d1c4e9', '#b39ddb', # deep purple
    '#e8eaf6', '#c5cae9', '#9fa8da', # indigo
    '#fffde7', '#fff9c4', '#fff59d', # yellow
    '#fff3e0', '#ffe0b2', '#ffcc80', # orange
    '#efebe9', '#d7ccc8', '#bcaaa4', # brown
  ]

  _colors = ['#263238', '#37474f', '#455a64', '#546e7a', '#607d8b', '#78909c','#90a4ae', '#b0bec5', '#cfd8dc', '#eceff1']
  _greys = ['#000000', '#212121', '#424242', '#616161', '#757575', '#9e9e9e', '#bdbdbd', '#e0e0e0', '#eeeeee',
            '#f5f5f5', '#FFFFFF']
  _warning, _danger, _success = ('#FFF3CD', '#e2ac00'), ("#F8D7DA", "#C00000"), ('#e8f2ef', '#3bb194')


class Grey(Theme.Theme):
  _colors = ['#263238', '#37474f', '#455a64', '#546e7a',
            '#607d8b', '#78909c', '#90a4ae', '#b0bec5',
            '#cfd8dc', '#eceff1']

  _greys = ['#273037', '#212121', '#273037', '#3F484D',
           '#757575', '#9e9e9e', '#bdbdbd', '#e0e0e0',
           '#eeeeee', '#f5f5f5', '#FFFFFF']

  _charts = ['#009999', '#336699', '#ffdcb9',
            '#cc99ff', '#b3d9ff', '#ffff99',
            '#000066', '#b2dfdb', '#80cbc4',
            '#ffebee', '#ffcdd2', '#ef9a9a', '#f3e5f5',
            '#e1bee7', '#ce93d8', '#ede7f6', '#d1c4e9',
            '#b39ddb', '#e8eaf6', '#c5cae9', '#9fa8da', '#fffde7', '#fff9c4', '#fff59d', '#fff3e0', '#ffe0b2',
            '#ffcc80', '#efebe9', '#d7ccc8', '#bcaaa4']

  _warning = ('#FFF3CD', '#e2ac00')
  _danger = ('#F8D7DA', '#C00000')
  _success = ('#e8f2ef', '#3bb194')




from . import HtmlTableConfig
from . import HtmlTablePivot
from . import HtmlTable
from . import HtlmTableDatatable
from . import HtmlTableTabulator
from . import HtmlTablePlotly
from . import HtmlTableAgGrid
from . import HtmlTableGoogle
